from __future__ import absolute_import

from datetime import datetime
from django.db import models
from krankshaft.api import API as APIBase
from krankshaft.auth import Auth as AuthBase
from krankshaft.authn import Authn
from krankshaft.authz import Authz as AuthzBase
from krankshaft.resource import DjangoModelResource
from tests.base import TestCaseNoDB
import json
import pytest

class Authz(AuthzBase):
    def is_authorized_object(self, request, authned, obj):
        return obj.__class__ is not ModelUnauthorized

class Auth(AuthBase):
    authn = Authn()
    authz = Authz(require_authned=False)

class API(APIBase):
    Auth = Auth

    def handler500(self, request, exc_info, error=None):
        raise

class ModelForeign(models.Model):
    char_indexed = models.CharField(max_length=20, db_index=True)
    created = models.DateTimeField(default=datetime.now)

class ModelForeign2(models.Model):
    char_indexed = models.CharField(max_length=20, db_index=True)

class ModelForeign3(models.Model):
    char_indexed = models.CharField(max_length=20, db_index=True)

class ModelForeignNoResource(models.Model):
    name = models.CharField(max_length=20, db_index=True)

class ModelForeignNoResourceForeign(models.Model):
    foreign = models.ForeignKey(ModelForeignNoResource)

class ModelHasForeign3(models.Model):
    foreign = models.ForeignKey(ModelForeign3)

class ModelMany(models.Model):
    char_indexed = models.CharField(max_length=20, db_index=True)

class ModelName(models.Model):
    name = models.CharField(max_length=20, db_index=True)

class ModelOther(models.Model):
    seconds = models.PositiveIntegerField(default=0)

class ModelUnauthorized(models.Model):
    name = models.CharField(max_length=20, db_index=True)

class Model(models.Model):
    char_indexed = models.CharField(max_length=20, db_index=True)
    foreign = models.ForeignKey(ModelForeign, null=True)
    manytomany = models.ManyToManyField(ModelMany)

class ModelAllowed(models.Model):
    name = models.CharField(max_length=20)

class ModelVersioned(models.Model):
    name = models.CharField(max_length=20)
    version = models.PositiveIntegerField(default=0)

    def save(self, *args, **kwargs):
        self.version += 1
        return super(ModelVersioned, self).save(*args, **kwargs)

@pytest.mark.django_db
class ResourceTest(TestCaseNoDB):
    def _pre_setup(self):
        api = self.api = API('v1', debug=True)
        api2 = self.api2 = API('v2', debug=True)

        @api
        class NameResource(DjangoModelResource):
            model = ModelName

        @api
        class ModelForeignResource(DjangoModelResource):
            model = ModelForeign
            excludes = ('created',)

        @api2
        class ModelForeign2Resource(DjangoModelResource):
            model = ModelForeign2
            use_location = True

        @api
        class ModelForeign3Resource(DjangoModelResource):
            model = ModelForeign3

        @api
        class ModelForeignNoResourceForeignResource(DjangoModelResource):
            model = ModelForeignNoResourceForeign
            name = 'modelforeignnoresourceforeign'

        @api
        class ModelHasForeign3Resource(DjangoModelResource):
            model = ModelHasForeign3

            def serialize_foreign(self, instance, field):
                resource = self.related_lookup(field)
                return resource.serialize(getattr(instance, field.name))

        @api
        class ModelManyResource(DjangoModelResource):
            model = ModelMany

        @api
        class ModelOtherResource(DjangoModelResource):
            model = ModelOther

            def deserialize_seconds(self, instance, field, data):
                return data['seconds'] / 10

            def serialize_seconds(self, instance, field):
                return instance.seconds * 10

        @api
        class ModelUnauthorizedResource(DjangoModelResource):
            model = ModelUnauthorized

        @api
        class ModelResource(DjangoModelResource):
            model = Model

        @api
        class ModelAllowedResource(DjangoModelResource):
            model = ModelAllowed
            allowed_methods = ('get',)

        @api
        class ModelAllowed2Resource(DjangoModelResource):
            model = ModelAllowed
            allowed_methods_endpoint = {
                'list': ('get',),
                'single': ('get',),
                'set': ('get',),
            }

        @api
        class ModelVersionedResource(DjangoModelResource):
            model = ModelVersioned
            version_field = 'version'

        # only to satisfy coverage (calling load() twice)
        for resource in self.api.registered_resources:
            resource.load()

        super(ResourceTest, self)._pre_setup()

    def tearDown(self):
        for model in [
            resource.model
            for resource in self.api.registered_resources
        ] + [
            resource.model
            for resource in self.api2.registered_resources
        ] + [
            ModelForeignNoResource,
        ]:
            model.objects.all().delete()

    def test_allowed(self):
        ModelAllowed.objects.create(id=1, name='allowed')
        ModelAllowed.objects.create(id=2, name='allowed2')

        for code, method in (
            (405, self.client.delete),
            (200, self.client.get),
            (405, self.client.post),
            (405, self.client.put),
        ):
            for endpoint, args in (
                ('list', ()),
                ('single', (1,)),
                ('set', ('1;2',)),
            ):
                response = method(
                    self.api.reverse('modelallowed_' + endpoint, args=args)
                )
                assert response.status_code == code

                response = method(
                    self.api.reverse('modelallowed2_' + endpoint, args=args)
                )
                assert response.status_code == code

    def test_delete_list(self):
        ModelForeign.objects.create(id=1, char_indexed='value')
        ModelForeign.objects.create(id=2, char_indexed='value1')
        ModelForeign.objects.create(id=3, char_indexed='value2')
        ModelForeign.objects.create(id=4, char_indexed='other')
        response = self.client.delete(
            self.api.reverse('modelforeign_list')
            + '?char_indexed__startswith=value'
        )

        assert response.status_code == 204
        assert response.content == ''
        assert ModelForeign.objects.count() == 1
        assert ModelForeign.objects.all()[0].id == 4

    def test_delete_set(self):
        ModelForeign.objects.create(id=1, char_indexed='value')
        ModelForeign.objects.create(id=2, char_indexed='value1')
        ModelForeign.objects.create(id=3, char_indexed='value2')
        ModelForeign.objects.create(id=4, char_indexed='other')
        response = self.client.delete(
            self.api.reverse('modelforeign_set', args=('1;3',))
        )

        assert response.status_code == 204
        assert response.content == ''
        assert ModelForeign.objects.count() == 2
        assert list(ModelForeign.objects.values_list('id', flat=True)) == [2, 4]

    def test_delete_single(self):
        ModelForeign.objects.create(id=1, char_indexed='value')
        response = self.client.delete(
            self.api.reverse('modelforeign_single', args=(1,))
        )

        assert response.status_code == 204
        assert response.content == ''

    def test_exclude_error(self):
        class ModelResource(DjangoModelResource):
            model = Model
            exclude = 'anything'

        self.assertRaises(ModelResource.Error, ModelResource)

    def test_fetch_invalid_id(self):
        resource, ids = self.api.resolve([
            self.api.reverse('modelother_single', args=('invalid',)),
        ])
        self.assertRaises(
            self.api.ValueIssue,
            resource.fetch,
            *ids
        )

    def test_get_list(self):
        ModelForeign.objects.create(id=1, char_indexed='value')
        ModelForeign.objects.create(id=2, char_indexed='value2')
        ModelForeign.objects.create(id=3, char_indexed='value3')
        ModelForeign.objects.create(id=4, char_indexed='other')
        response = self.client.get(
            self.api.reverse('modelforeign_list')
            + '?char_indexed__startswith=value&order_by=id'
        )

        assert response.status_code == 200
        assert response['Content-Type'] == 'application/json; charset=utf-8'
        assert json.loads(response.content) == {
            'meta': {
                'limit': None,
                'next': None,
                'offset': 0,
                'previous': None
            },
            'objects': [
                {
                    'id': 1,
                    'char_indexed': 'value',
                    'resource_uri': '/api/v1/modelforeign/1/'
                },
                {
                    'id': 2,
                    'char_indexed': 'value2',
                    'resource_uri': '/api/v1/modelforeign/2/'
                },
                {
                    'id': 3,
                    'char_indexed': 'value3',
                    'resource_uri': '/api/v1/modelforeign/3/'
                },
            ]
        }

    def test_get_list_paginate(self):
        ModelForeign.objects.create(id=1, char_indexed='value')
        ModelForeign.objects.create(id=2, char_indexed='value2')
        ModelForeign.objects.create(id=3, char_indexed='value3')
        ModelForeign.objects.create(id=4, char_indexed='other')
        response = self.client.get(
            self.api.reverse('modelforeign_list')
            + '?char_indexed__startswith=value&order_by=id&limit=1'
        )

        assert response.status_code == 200
        assert response['Content-Type'] == 'application/json; charset=utf-8'
        assert json.loads(response.content) == {
            'meta': {
                'limit': 1,
                'next': '/api/v1/modelforeign/?order_by=id&char_indexed__startswith=value&limit=1&offset=1',
                'offset': 0,
                'previous': None
            },
            'objects': [
                {
                    'id': 1,
                    'char_indexed': 'value',
                    'resource_uri': '/api/v1/modelforeign/1/'
                },
            ]
        }

        response = self.client.get(
            '/api/v1/modelforeign/?order_by=id&char_indexed__startswith=value&limit=1&offset=1',
        )

        assert response.status_code == 200
        assert response['Content-Type'] == 'application/json; charset=utf-8'
        assert json.loads(response.content) == {
            'meta': {
                'limit': 1,
                'next': '/api/v1/modelforeign/?order_by=id&char_indexed__startswith=value&limit=1&offset=2',
                'offset': 1,
                'previous': '/api/v1/modelforeign/?order_by=id&char_indexed__startswith=value&limit=1&offset=0',
            },
            'objects': [
                {
                    'id': 2,
                    'char_indexed': 'value2',
                    'resource_uri': '/api/v1/modelforeign/2/'
                },
            ]
        }

    def test_get_set(self):
        ModelForeign.objects.create(id=1, char_indexed='value')
        ModelForeign.objects.create(id=2, char_indexed='value2')
        ModelForeign.objects.create(id=3, char_indexed='value3')
        ModelForeign.objects.create(id=4, char_indexed='other')
        response = self.client.get(
            self.api.reverse('modelforeign_set', args=('1;3',))
        )

        assert response.status_code == 200
        assert response['Content-Type'] == 'application/json; charset=utf-8'
        assert json.loads(response.content) == {
            'objects': [
                {
                    'id': 1,
                    'char_indexed': 'value',
                    'resource_uri': '/api/v1/modelforeign/1/'
                },
                {
                    'id': 3,
                    'char_indexed': 'value3',
                    'resource_uri': '/api/v1/modelforeign/3/'
                },
            ]
        }

    def test_get_set_invalid_id(self):
        response = self.client.get(
            self.api.reverse('modelforeign_set', args=('invalid',))
        )

        assert response.status_code == 400
        assert response['Content-Type'] == 'application/json; charset=utf-8'
        assert json.loads(response.content) == {
            'error': 'Invalid ID for model ModelForeign',
            'invalid': [u"invalid literal for int() with base 10: 'invalid'"],
        }

    def test_get_set_missing(self):
        ModelForeign.objects.create(id=1, char_indexed='value')
        ModelForeign.objects.create(id=2, char_indexed='value2')
        ModelForeign.objects.create(id=4, char_indexed='other')
        response = self.client.get(
            self.api.reverse('modelforeign_set', args=('1;3',))
        )

        assert response.status_code == 404
        assert response['Content-Type'] == 'application/json; charset=utf-8'
        assert json.loads(response.content) == {
            'error': 'Missing some requested objects',
            'missing': [3],
        }

    def test_get_set_trailing_separator(self):
        response = self.client.get(
            self.api.reverse('modelforeign_set', args=('1;2;',))
        )

        assert response.status_code == 400
        assert response['Content-Type'] == 'application/json; charset=utf-8'
        assert json.loads(response.content) == {
            'error': 'Invalid ID for model ModelForeign',
            'invalid': [u"invalid literal for int() with base 10: ''"],
        }

    def test_get_single(self):
        ModelForeign.objects.create(id=1, char_indexed='value')
        response = self.client.get(
            self.api.reverse('modelforeign_single', args=(1,))
        )

        assert response.status_code == 200
        assert response['Content-Type'] == 'application/json; charset=utf-8'
        assert json.loads(response.content) == {
            'id': 1,
            'char_indexed': 'value',
            'resource_uri': '/api/v1/modelforeign/1/'
        }

    def test_get_single_foreign_null(self):
        Model.objects.create(id=1, foreign=None)

        response = self.client.get(
            self.api.reverse('model_single', args=(1,)),
        )
        assert response.status_code == 200
        assert json.loads(response.content) == {
            'id': 1,
            'char_indexed': '',
            'foreign': None,
            'foreign_id': None,
            'manytomany': [],
            'manytomany_id': [],
            'resource_uri': '/api/v1/model/1/',
        }

    def test_get_single_missing(self):
        response = self.client.get(
            self.api.reverse('modelforeign_single', args=(1,))
        )

        assert response.status_code == 404
        assert response.content == ''

    def test_get_single_no_foreign_resource(self):
        ModelForeignNoResource.objects.create(id=1, name='foreign')
        ModelForeignNoResourceForeign.objects.create(id=1, foreign_id=1)

        response = self.client.get(
            self.api.reverse('modelforeignnoresourceforeign_single', args=(1,))
        )
        assert response.status_code == 200
        assert response['Content-Type'] == 'application/json; charset=utf-8'
        assert json.loads(response.content) == {
            'id': 1,
            'foreign': 1,
            'foreign_id': 1,
            'resource_uri': '/api/v1/modelforeignnoresourceforeign/1/'
        }

    def test_get_single_defer_char_indexed(self):
        ModelForeign.objects.create(id=1, char_indexed='value')
        response = self.client.get(
            self.api.reverse('modelforeign_single', args=(1,))
            + '?defer=char_indexed'
        )

        assert response.status_code == 200
        assert response['Content-Type'] == 'application/json; charset=utf-8'
        assert json.loads(response.content) == {
            'id': 1,
            'resource_uri': '/api/v1/modelforeign/1/'
        }

    def test_get_single_only_id(self):
        ModelForeign.objects.create(id=1, char_indexed='value')
        response = self.client.get(
            self.api.reverse('modelforeign_single', args=(1,))
            + '?only=id'
        )

        assert response.status_code == 200
        assert response['Content-Type'] == 'application/json; charset=utf-8'
        assert json.loads(response.content) == {
            'id': 1,
            'resource_uri': '/api/v1/modelforeign/1/'
        }

    def test_get_single_unauthorized(self):
        ModelUnauthorized.objects.create(id=1, name='notauthzed')
        response = self.client.get(
            self.api.reverse('modelunauthorized_single', args=(1,))
        )

        assert response.status_code == 401
        assert response.content == ''

    def test_hook_serialize_field(self):
        ModelOther.objects.create(id=1, seconds=10)
        response = self.client.get(
            self.api.reverse('modelother_single', args=(1,))
        )

        assert response.status_code == 200
        assert response['Content-Type'] == 'application/json; charset=utf-8'
        assert json.loads(response.content) == {
            'id': 1,
            'seconds': 100,
            'resource_uri': '/api/v1/modelother/1/'
        }

        response = self.client.put(
            self.api.reverse('modelother_single', args=(1,)),
            json.dumps({'seconds': 10}),
            content_type='application/json'
        )
        assert response.status_code == 200
        assert response['Content-Type'] == 'application/json; charset=utf-8'
        assert json.loads(response.content) == {
            'id': 1,
            'seconds': 10,
            'resource_uri': '/api/v1/modelother/1/'
        }

        other = ModelOther.objects.get(id=1)
        assert other.seconds == 1

    def test_many_to_many_update(self):
        foreign = ModelForeign.objects.create(id=1, char_indexed='value')
        many = ModelMany.objects.create(id=1, char_indexed='value')
        many2 = ModelMany.objects.create(id=2, char_indexed='value2')
        instance = Model.objects.create(id=1, char_indexed='value', foreign=foreign)
        instance.manytomany.add(many)

        response = self.client.get(self.api.reverse('model_single', args=(1,)))
        assert response.status_code == 200
        assert response['Content-Type'] == 'application/json; charset=utf-8'
        assert json.loads(response.content) == {
            'id': 1,
            'char_indexed': 'value',
            'foreign': '/api/v1/modelforeign/1/',
            'foreign_id': 1,
            'manytomany': ['/api/v1/modelmany/1/'],
            'manytomany_id': [1],
            'resource_uri': '/api/v1/model/1/'
        }

        response = self.client.put(
            self.api.reverse('model_single', args=(1,)),
            json.dumps({'manytomany': [
                '/api/v1/modelmany/2/',
            ]}),
            content_type='application/json'
        )
        assert response.status_code == 200
        assert response['Content-Type'] == 'application/json; charset=utf-8'
        assert json.loads(response.content) == {
            'id': 1,
            'char_indexed': 'value',
            'foreign': '/api/v1/modelforeign/1/',
            'foreign_id': 1,
            'manytomany': ['/api/v1/modelmany/2/'],
            'manytomany_id': [2],
            'resource_uri': '/api/v1/model/1/'
        }

    def test_post_list(self):
        response = self.client.post(
            self.api.reverse('modelforeign_list'),
            json.dumps({'id': 1, 'char_indexed': 'posted'}),
            content_type='application/json'
        )

        assert response.status_code == 200
        assert response['Content-Type'] == 'application/json; charset=utf-8'
        assert json.loads(response.content) == {
            'id': 1,
            'char_indexed': 'posted',
            'resource_uri': '/api/v1/modelforeign/1/'
        }

        instance = ModelForeign.objects.get(id=1)
        assert instance.char_indexed == 'posted'
        assert instance.created is not None

    def test_post_list_location(self):
        response = self.client.post(
            self.api2.reverse('modelforeign2_list'),
            json.dumps({'id': 1, 'char_indexed': 'posted'}),
            content_type='application/json'
        )

        assert response.status_code == 204
        assert response.content == ''
        assert response['Location'].endswith(self.api2.reverse('modelforeign2_single', args=(1,)))

        response = self.client.get(response['Location'])
        assert response.status_code == 200
        assert response['Content-Type'] == 'application/json; charset=utf-8'
        assert json.loads(response.content) == {
            'id': 1,
            'char_indexed': 'posted',
            'resource_uri': '/api/v2/modelforeign2/1/'
        }

        instance = ModelForeign2.objects.get(id=1)
        assert instance.char_indexed == 'posted'

    def test_put_list(self):
        ModelForeign.objects.create(id=1, char_indexed='value')
        ModelForeign.objects.create(id=2, char_indexed='value2')
        ModelForeign.objects.create(id=3, char_indexed='value3')
        ModelForeign.objects.create(id=4, char_indexed='other')
        response = self.client.put(
            self.api.reverse('modelforeign_list')
                + '?char_indexed__startswith=value&order_by=id',
            json.dumps({'char_indexed': 'massupdate'}),
            content_type='application/json'
        )

        assert response.status_code == 200
        assert response['Content-Type'] == 'application/json; charset=utf-8'
        assert json.loads(response.content) == {
            'meta': {
                'limit': None,
                'next': None,
                'offset': 0,
                'previous': None
            },
            'objects': [
                {
                    'id': 1,
                    'char_indexed': 'massupdate',
                    'resource_uri': '/api/v1/modelforeign/1/'
                },
                {
                    'id': 2,
                    'char_indexed': 'massupdate',
                    'resource_uri': '/api/v1/modelforeign/2/'
                },
                {
                    'id': 3,
                    'char_indexed': 'massupdate',
                    'resource_uri': '/api/v1/modelforeign/3/'
                },
            ],
        }

        id1 = ModelForeign.objects.get(id=1)
        id2 = ModelForeign.objects.get(id=2)
        id3 = ModelForeign.objects.get(id=3)
        id4 = ModelForeign.objects.get(id=4)
        assert id1.char_indexed == 'massupdate'
        assert id2.char_indexed == 'massupdate'
        assert id3.char_indexed == 'massupdate'
        assert id4.char_indexed == 'other'

    def test_put_list_location(self):
        ModelForeign2.objects.create(id=1, char_indexed='value')
        ModelForeign2.objects.create(id=2, char_indexed='value2')
        ModelForeign2.objects.create(id=3, char_indexed='value3')
        ModelForeign2.objects.create(id=4, char_indexed='other')
        response = self.client.put(
            self.api2.reverse('modelforeign2_list')
                + '?char_indexed__startswith=value&order_by=id',
            json.dumps({'char_indexed': 'massupdate'}),
            content_type='application/json'
        )

        assert response.status_code == 204
        assert response.content == ''
        assert response['Location'].endswith(
            self.api2.reverse('modelforeign2_set', args=('1;2;3', ))
        )

        response = self.client.get(response['Location'])
        assert response.status_code == 200
        assert response['Content-Type'] == 'application/json; charset=utf-8'
        assert json.loads(response.content) == {
            'objects': [
                {
                    'id': 1,
                    'char_indexed': 'massupdate',
                    'resource_uri': '/api/v2/modelforeign2/1/'
                },
                {
                    'id': 2,
                    'char_indexed': 'massupdate',
                    'resource_uri': '/api/v2/modelforeign2/2/'
                },
                {
                    'id': 3,
                    'char_indexed': 'massupdate',
                    'resource_uri': '/api/v2/modelforeign2/3/'
                },
            ],
        }

        id1 = ModelForeign2.objects.get(id=1)
        id2 = ModelForeign2.objects.get(id=2)
        id3 = ModelForeign2.objects.get(id=3)
        id4 = ModelForeign2.objects.get(id=4)
        assert id1.char_indexed == 'massupdate'
        assert id2.char_indexed == 'massupdate'
        assert id3.char_indexed == 'massupdate'
        assert id4.char_indexed == 'other'

    def test_put_set(self):
        ModelForeign.objects.create(id=1, char_indexed='value')
        ModelForeign.objects.create(id=2, char_indexed='value2')
        ModelForeign.objects.create(id=3, char_indexed='value3')
        ModelForeign.objects.create(id=4, char_indexed='other')
        response = self.client.put(
            self.api.reverse('modelforeign_set', args=('1;3',)),
            json.dumps([
                {'id': 1, 'char_indexed': 'setupdate1'},
                {'id': 3, 'char_indexed': 'setupdate3'},
            ]),
            content_type='application/json'
        )

        assert response.status_code == 200
        assert response['Content-Type'] == 'application/json; charset=utf-8'
        assert json.loads(response.content) == {
            'objects': [
                {
                    'id': 1,
                    'char_indexed': 'setupdate1',
                    'resource_uri': '/api/v1/modelforeign/1/'
                },
                {
                    'id': 3,
                    'char_indexed': 'setupdate3',
                    'resource_uri': '/api/v1/modelforeign/3/'
                },
            ],
        }

        id1 = ModelForeign.objects.get(id=1)
        id3 = ModelForeign.objects.get(id=3)
        assert id1.char_indexed == 'setupdate1'
        assert id3.char_indexed == 'setupdate3'

    def test_put_set_invalid_format(self):
        ModelForeign.objects.create(id=1, char_indexed='value')
        ModelForeign.objects.create(id=3, char_indexed='value3')
        response = self.client.put(
            self.api.reverse('modelforeign_set', args=('1;3',)),
            json.dumps({}),
            content_type='application/json'
        )

        assert response.status_code == 422
        assert response['Content-Type'] == 'application/json; charset=utf-8'
        assert json.loads(response.content) == {
            'error': 'Supplied data was invalid',
            'invalid': ["Unexpected type, expected <type 'list'>: <type 'dict'>"],
        }

    def test_put_set_location(self):
        ModelForeign2.objects.create(id=1, char_indexed='value')
        ModelForeign2.objects.create(id=2, char_indexed='value2')
        ModelForeign2.objects.create(id=3, char_indexed='value3')
        ModelForeign2.objects.create(id=4, char_indexed='other')
        response = self.client.put(
            self.api2.reverse('modelforeign2_set', args=('1;3',)),
            json.dumps([
                {'id': 1, 'char_indexed': 'setupdate1'},
                {'id': 3, 'char_indexed': 'setupdate3'},
            ]),
            content_type='application/json'
        )

        assert response.status_code == 204
        assert response.content == ''
        assert response['Location'].endswith(
            self.api2.reverse('modelforeign2_set', args=('1;3', ))
        )

        response = self.client.get(response['Location'])
        assert response.status_code == 200
        assert response['Content-Type'] == 'application/json; charset=utf-8'
        assert json.loads(response.content) == {
            'objects': [
                {
                    'id': 1,
                    'char_indexed': 'setupdate1',
                    'resource_uri': '/api/v2/modelforeign2/1/'
                },
                {
                    'id': 3,
                    'char_indexed': 'setupdate3',
                    'resource_uri': '/api/v2/modelforeign2/3/'
                },
            ],
        }

        id1 = ModelForeign2.objects.get(id=1)
        id3 = ModelForeign2.objects.get(id=3)
        assert id1.char_indexed == 'setupdate1'
        assert id3.char_indexed == 'setupdate3'

    def test_put_set_must_include_id_in_representation(self):
        ModelForeign.objects.create(id=1, char_indexed='value')
        ModelForeign.objects.create(id=2, char_indexed='value2')
        ModelForeign.objects.create(id=3, char_indexed='value3')
        ModelForeign.objects.create(id=4, char_indexed='other')
        response = self.client.put(
            self.api.reverse('modelforeign_set', args=('1;3',)),
            json.dumps([
                {'char_indexed': 'setupdate1'},
                {'char_indexed': 'setupdate3'},
            ]),
            content_type='application/json'
        )

        assert response.status_code == 400
        assert response['Content-Type'] == 'application/json; charset=utf-8'
        assert json.loads(response.content) == {
            'error': 'You must supply the primary key with each object',
        }

    def test_put_single(self):
        ModelForeign.objects.create(id=1, char_indexed='value')
        response = self.client.put(
            self.api.reverse('modelforeign_single', args=(1,)),
            json.dumps({'char_indexed': 'updated'}),
            content_type='application/json'
        )

        assert response.status_code == 200
        assert response['Content-Type'] == 'application/json; charset=utf-8'
        assert json.loads(response.content) == {
            'id': 1,
            'char_indexed': 'updated',
            'resource_uri': '/api/v1/modelforeign/1/'
        }

        instance = ModelForeign.objects.get(id=1)
        assert instance.char_indexed == 'updated'
        assert instance.created is not None

    def test_put_single_location(self):
        ModelForeign2.objects.create(id=1, char_indexed='value')
        response = self.client.put(
            self.api2.reverse('modelforeign2_single', args=(1,)),
            json.dumps({'char_indexed': 'updated'}),
            content_type='application/json'
        )

        assert response.status_code == 204
        assert response.content == ''
        assert response['Location'].endswith(self.api2.reverse('modelforeign2_single', args=(1,)))

        response = self.client.get(response['Location'])
        assert response.status_code == 200
        assert response['Content-Type'] == 'application/json; charset=utf-8'
        assert json.loads(response.content) == {
            'id': 1,
            'char_indexed': 'updated',
            'resource_uri': '/api/v2/modelforeign2/1/'
        }

        instance = ModelForeign2.objects.get(id=1)
        assert instance.char_indexed == 'updated'

    def test_put_single_invalid_manytomany(self):
        ModelForeign.objects.create(id=1)
        Model.objects.create(id=1, foreign_id=1)

        response = self.client.put(
            self.api.reverse('model_single', args=(1,)),
            json.dumps({
                'manytomany': [
                    '/api/v1/modelforeign/invalid/',
                ]
            }),
            content_type='application/json'
        )
        assert response.status_code == 422
        assert response['Content-Type'] == 'application/json; charset=utf-8'
        assert json.loads(response.content) == {
            'error': 'Supplied data was invalid',
            'invalid': {
                'manytomany': ["Unable to resolve related: [u'invalid'], Unexpected resource found: modelforeign (expected modelmany)"]
            },
        }

    def test_resource_deserialize(self):
        response = self.client.post(
            self.api.reverse('modelmany_list'),
            json.dumps({'char_indexed': 'value'}),
            content_type='application/json'
        )
        assert response.status_code == 200
        assert response['Content-Type'] == 'application/json; charset=utf-8'
        assert json.loads(response.content) == {
            'id': 1,
            'char_indexed': 'value',
            'resource_uri': '/api/v1/modelmany/1/',
        }

        instance_many = ModelMany.objects.get(id=1)
        assert instance_many.id == 1
        assert instance_many.char_indexed == 'value'

        response = self.client.post(
            self.api.reverse('modelforeign_list'),
            json.dumps({'char_indexed': 'value'}),
            content_type='application/json'
        )
        assert response.status_code == 200
        assert response['Content-Type'] == 'application/json; charset=utf-8'
        assert json.loads(response.content) == {
            'id': 1,
            'char_indexed': 'value',
            'resource_uri': '/api/v1/modelforeign/1/',
        }

        instance_foreign = ModelForeign.objects.get(id=1)
        assert instance_foreign.id == 1
        assert instance_foreign.char_indexed == 'value'

        response = self.client.post(
            self.api.reverse('model_list'),
            json.dumps({
                'char_indexed': 'value',
                'foreign': self.api.reverse('modelforeign_single', args=(1,)),
                'manytomany': [
                    self.api.reverse('modelmany_single', args=(1,)),
                ]
            }),
            content_type='application/json'
        )
        assert response.status_code == 200
        assert response['Content-Type'] == 'application/json; charset=utf-8'
        assert json.loads(response.content) == {
            'id': 1,
            'char_indexed': 'value',
            'foreign': '/api/v1/modelforeign/1/',
            'foreign_id': 1,
            'manytomany': [
                '/api/v1/modelmany/1/',
            ],
            'manytomany_id': [1],
            'resource_uri': '/api/v1/model/1/',
        }

    def test_resource_deserialize_invalid_resource_uri(self):
        ModelForeign.objects.create(id=1, char_indexed='value')
        Model.objects.create(id=1, char_indexed='value', foreign_id=1)
        response = self.client.put(
            self.api.reverse('model_single', args=(1,)),
            json.dumps({
                'foreign': '/api/v1/model/1/',
            }),
            content_type='application/json'
        )

        assert response.status_code == 422
        assert response['Content-Type'] == 'application/json; charset=utf-8'
        assert json.loads(response.content) == {
            'error': 'Supplied data was invalid',
            'invalid': {
                'foreign': ["Unable to resolve related: [u'1'], Unexpected resource found: model (expected modelforeign)"]
            }
        }

    def test_resource_resolve(self):
        resource, ids = self.api.resolve([
            self.api.reverse('modelother_single', args=(1,)),
        ])
        self.assertRaises(
            ModelOther.DoesNotExist,
            resource.fetch,
            *ids
        )

    def test_resource_serialize(self):
        instance_foreign = ModelForeign.objects.create(id=1, char_indexed='value')
        instance = Model.objects.create(id=1, char_indexed='value', foreign=instance_foreign)
        instance.manytomany.add(*[
            ModelMany.objects.create(id=1, char_indexed='value'),
            ModelMany.objects.create(id=2, char_indexed='value2'),
            ModelMany.objects.create(id=3, char_indexed='value3'),
        ])

        response = self.client.get(self.api.reverse('model_single', args=(1,)))
        assert response.status_code == 200
        assert response['Content-Type'] == 'application/json; charset=utf-8'
        assert json.loads(response.content) == {
            'id': 1,
            'char_indexed': 'value',
            'foreign': '/api/v1/modelforeign/1/',
            'foreign_id': 1,
            'manytomany': [
                '/api/v1/modelmany/1/',
                '/api/v1/modelmany/2/',
                '/api/v1/modelmany/3/',
            ],
            'manytomany_id': [1,2,3],
            'resource_uri': '/api/v1/model/1/',
        }

    def test_serialize_foreign3(self):
        ModelForeign3.objects.create(id=1, char_indexed='value')
        ModelHasForeign3.objects.create(id=1, foreign_id=1)

        response = self.client.get(
            self.api.reverse('modelhasforeign3_single', args=(1,))
        )
        assert json.loads(response.content) == {
            'id': 1,
            'foreign': {
                'id': 1,
                'char_indexed': 'value',
                'resource_uri': '/api/v1/modelforeign3/1/',
            },
            'resource_uri': '/api/v1/modelhasforeign3/1/',
        }

    def test_version_field(self):
        ModelVersioned.objects.create(id=1, name='initial')

        resource_uri = self.api.reverse('modelversioned_single', args=(1,))
        response = self.client.put(
            resource_uri,
            json.dumps({
                'name': 'first',
                'version': 1,
            }),
            content_type='application/json'
        )
        assert response.status_code == 200
        assert json.loads(response.content) == {
            'id': 1,
            'name': 'first',
            'version': 2,
            'resource_uri': resource_uri,
        }

        response = self.client.put(
            resource_uri,
            json.dumps({
                'name': 'invalid',
                'version': 1,
            }),
            content_type='application/json'
        )
        assert response.status_code == 409
        assert response.content == ''

        response = self.client.put(
            resource_uri,
            json.dumps({
                'name': 'second',
                'version': 2,
            }),
            content_type='application/json'
        )
        assert response.status_code == 200
        assert json.loads(response.content) == {
            'id': 1,
            'name': 'second',
            'version': 3,
            'resource_uri': resource_uri,
        }

    def test_version_field_missing_version_field(self):
        ModelVersioned.objects.create(id=1, name='initial')

        resource_uri = self.api.reverse('modelversioned_single', args=(1,))
        response = self.client.put(
            resource_uri,
            json.dumps({
                'name': 'first',
            }),
            content_type='application/json'
        )
        assert response.status_code == 400
        assert json.loads(response.content) == {
            'error': 'The "version" field must be specified',
        }

    def test_version_field_put_set(self):
        ModelVersioned.objects.create(id=1, name='initial')
        ModelVersioned.objects.create(id=2, name='initial')

        response = self.client.put(
            self.api.reverse('modelversioned_set', args=('1;2',)),
            json.dumps([
                {
                    'id': 1,
                    'name': 'invalid',
                },
                {
                    'id': 2,
                    'name': 'invalid',
                },
            ]),
            content_type='application/json'
        )
        assert response.status_code == 400
        assert json.loads(response.content) == {
            'invalid': {
                '1': {'error': 'The "version" field must be specified'},
                '2': {'error': 'The "version" field must be specified'},
            }
        }

        response = self.client.put(
            self.api.reverse('modelversioned_set', args=('1;2',)),
            json.dumps([
                {
                    'id': 1,
                    'name': 'invalid',
                    'version': 2,
                },
                {
                    'id': 2,
                    'name': 'invalid',
                    'version': 2,
                },
            ]),
            content_type='application/json'
        )
        assert response.status_code == 409
        assert response.content == ''

        response = self.client.put(
            self.api.reverse('modelversioned_set', args=('1;2',)),
            json.dumps([
                {
                    'id': 1,
                    'name': 'invalid',
                },
                {
                    'id': 2,
                    'name': 'invalid',
                    'version': 2,
                },
            ]),
            content_type='application/json'
        )
        assert response.status_code == 400
        assert json.loads(response.content) == {
            'error': 'Mixed error codes',
            'invalid': {
                '1': {
                    'code': 400,
                    'error': 'The "version" field must be specified'
                },
                '2': {'code': 409},
            }
        }

    @property
    def urls(self):
        from django.conf.urls import include, url

        return self.make_urlconf(
            url('^api/', include(self.api.urls)),
            url('^api/', include(self.api2.urls)),
        )



# TODO OAuth1/2

# TODO Convert "expect" to "method"

import logging

log = logging.getLogger(__name__)

class Authn(object):
    '''
    Basic interface for any authenticator.  Always returns the request as
    not authenticated.
    '''
    expect = None

    def __init__(self, realm=None):
        self.use_realm = realm

    def authenticate(self, request):
        return None

    def authorization(self, request):
        return ''

    def can_handle(self, request):
        return bool(self.parse_expect(request))

    def challenge(self, request, response):
        return response

    def find_realm(self):
        return ''

    def is_valid(self, authned):
        return True

    def parse(self, request):
        credentials = self.parse_expect(request)
        if not credentials:
            return None

        try:
            identifer, credential = credentials.split(':', 1)
        except ValueError:
            return None

        if not (identifer and credential):
            return None

        return (identifer, credential)

    def parse_expect(self, request):
        header = self.authorization(request)
        if not header:
            return None

        try:
            method, credentials = header.split(None, 1)
        except ValueError:
            return None

        expect = self.expect
        if not expect:
            return credentials

        if isinstance(expect, (tuple, list)):
            if method.lower() not in expect:
                return None

        elif method.lower() != expect:
            return None

        return credentials

    @property
    def realm(self):
        return self.use_realm or self.find_realm()

    @property
    def secret(self):
        return ''

class AuthnDjango(Authn):
    '''
    Meant to be subclassed by Django Authenticators.  Will not work by itself.
    '''
    def authenticate(self, **credentials):
        from django.contrib.auth import authenticate as auth_authenticate
        return auth_authenticate(**credentials)

    def authorization(self, request):
        return request.META.get('HTTP_AUTHORIZATION', '')

    def find_realm(self):
        from django.contrib.sites.models import Site
        return Site.objects.get_current().name

    def is_valid(self, authned):
        return authned.user.is_active

    @property
    def secret(self):
        from django.conf import settings
        return settings.SECRET_KEY

class AuthnDjangoAPIToken(AuthnDjango):
    '''AuthnDjangoAPIToken(APIToken)

    An authenticator that uses a model that sub-classes the
    krankshaft.models.APITokenBase class as a form of authentication. Intended
    to make it possible to have credentials for a user separate from the
    password that are easily invalidated in one way or another.
    '''
    expect = ('apitoken', 'apikey')

    def __init__(self, model, **kwargs):
        super(AuthnDjangoAPIToken, self).__init__(**kwargs)
        self.model = model

    def authenticate(self, request):
        try:
            owner, token = self.parse(request)
        except TypeError:
            return None

        try:
            return self.model.get(owner, token)
        except (self.model.DoesNotExist, self.model.MultipleObjectsReturned):
            return None

    def challenge(self, request, response):
        response['WWW-Authenticate'] = 'APIToken realm="%s"' % self.realm
        return response

    def is_valid(self, authned):
        return authned.is_valid()

class AuthnDjangoBasic(AuthnDjango):
    '''
    Basic authentication using Django's Auth framework.
    '''
    expect = 'basic'

    def authenticate(self, request):
        try:
            username, password = self.parse(request)
        except TypeError:
            return None

        return super(AuthnDjangoBasic, self).authenticate(
            username=username,
            password=password,
        )

    def challenge(self, request, response):
        response['WWW-Authenticate'] = 'Basic realm="%s"' % self.realm
        return response

class AuthnDjangoMiddleware(AuthnDjango):
    '''
    An authenticator that passes through any authentication already provided
    by middleware.  Typically usage could be with Session middleware.
    Specifically, it uses whatever user is attached to the request object.

    note: CSRF validation is done to ensure a certain level of safety that the
    request came from a page served by the application.
    See: https://docs.djangoproject.com/en/dev/ref/contrib/csrf/#ajax
    '''
    def authenticate(self, request):
        from django.middleware.csrf import CsrfViewMiddleware

        # return reason not response
        class CSRF(CsrfViewMiddleware):
            def _reject(self, request, reason):
                return reason

        user = getattr(request, 'user', None)
        if not user:
            return None

        if not user.is_authenticated():
            return None

        reason = CSRF().process_view(request, None, (), {})
        if reason:
            log.warn('Authn failed due to CSRF verification failure')
            return None

        return user

class AuthnedInterface(object):
    '''
    Proxy to an authned object to give a common interface to objects returned
    from Authn.authenticate() or any of its subclasses.
    '''
    def __init__(self, authned):
        self.authned = authned

    @property
    def id(self):
        return self.authned.id

    def is_valid(self):
        is_valid = None

        if hasattr(self.authned, 'is_valid'):
            is_valid = self.authned.is_valid

        if is_valid and hasattr(is_valid, '__call__'):
            return is_valid()

        return bool(is_valid)

    @property
    def name(self):
        return self.authned.__class__.__name__

    @property
    def user(self):
        if hasattr(self.authned, 'user'):
            return self.authned.user
        return self.authned # assume we're an interface to a user



class Authz(object):
    '''
    Basic interface for any authorizor.  Always returns the request as
    authorized.

    Options:
        require_authned: require that clients are authenticated
    '''
    methods_create = ('post', )
    methods_read = (
        'get',
        'head',
        'options',
    )
    methods_update = ('put', )
    methods_delete = ('delete', )

    def __init__(self, require_authned=True):
        self.require_authned = require_authned

    def is_authorized_object(self, request, authned, obj):
        return True

    def is_authorized_request(self, request, authned):
        if self.require_authned:
            return bool(authned)
        else:
            return True

    def limit(self, request, authned, query):
        return query

class AuthzDjango(Authz):
    '''
    Django specific authorization based on authenticated user permissions
    and if defined, passes authorization to model's is_authorized() method.
    This can be used to implement object-level authorization.

    Options:
        perms: enable model level permission checking (default: True)
    '''
    def __init__(self, perms=True, **kwargs):
        super(AuthzDjango, self).__init__(**kwargs)
        self.perms = perms

    def is_authorized_object(self, request, authned, obj):
        if self.perms:
            meta = obj._meta
            method = request.method.lower()
            perm = None
            if method in self.methods_read:
                pass

            elif method in self.methods_create:
                perm = '%s.%s' % (meta.app_label, meta.get_add_permission())

            elif method in self.methods_update:
                perm = '%s.%s' % (meta.app_label, meta.get_change_permission())

            elif method in self.methods_delete:
                perm = '%s.%s' % (meta.app_label, meta.get_delete_permission())

            else:
                # unhandled methods are not authorized
                return False

            # allow read methods through but verify authenticated has permission
            # to perform the given action on the object
            if perm and not authned.user.has_perm(perm):
                return False

        return self.is_authorized_object_forward(request, authned, obj)

    def is_authorized_object_default(self, request, authned, obj):
        return False

    def is_authorized_object_forward(self, request, authned, obj):
        obj_authz = getattr(obj, 'is_authorized', None)
        if obj_authz:
            return obj_authz(request, authned)

        else:
            return self.is_authorized_object_default(request, authned, obj)

class AuthzDjangoAnonRead(AuthzDjango):
    '''
    Like AuthzDjango, but explicitly allows read requests through without
    authentication.
    '''
    def is_authorized_request(self, request, authned):
        if authned:
            return True

        else:
            return request.method.lower() in self.methods_read

    def is_authorized_object_default(self, request, authned, obj):
        return request.method.lower() in self.methods_read

    def is_authorized_object_forward(self, request, authned, obj):
        if request.method.lower() in self.methods_read:
            return True
        else:
            return super(AuthzDjangoAnonRead, self) \
                .is_authorized_object_forward(request, authned, obj)

class AuthzReadonly(Authz):
    '''
    Read only authorization.  Only HTTP methods considered to be read-only
    are authorized.
    '''
    def is_authorized_request(self, request, authned):
        return \
            super(AuthzReadonly, self).is_authorized_request(request, authned) \
            and request.method.lower() in self.methods_read

    def is_authorized_object(self, request, authned, obj):
        return self.is_authorized_request(request, authned)



class KrankshaftError(Exception):
    pass

class Abort(KrankshaftError):
    def __init__(self, response):
        self.response = response

class ExpectedIssue(KrankshaftError):
    pass

class InvalidOptions(KrankshaftError):
    pass

class QueryIssues(KrankshaftError):
    pass

class ResolveError(KrankshaftError):
    pass

class ValueIssue(KrankshaftError):
    def __init__(self, errors):
        self.errors = errors

    def __str__(self):
        from pprint import pformat
        msg = pformat(self.errors)
        if '\n' in msg:
            msg = '\n' + msg
        return msg



'''
valid - helper routines for validating/cleaning data from untrusted inputs

The pattern and reason for this module are quite simple.  Make it easy to
define an expected structure and clean incoming data based on that structure.

To write a validator, you need only be able to write a function that raises
ValueError with a valuable message on data that is not valid.  Then using
expect(), you can reuse that function to properly validate more complex data
structures.  If a value is valid, but needs transformation (ie: '1' -> 1,
convert from string 1 to integer 1) simply return the cleaned value from the
function.  A simple validator looks like this:

    def validator(value, expect):
        ... raise ValueError if there are issues
        ... if you need to recall expect on a new structure, then:
        ...     expect(expected, newvalue)
        return value

Ideally, you can simply use something like this:

    from krankshaft import valid

    try:
        clean = api.expect(
            {
                'count': valid.int,
                'name': valid.string_max_length(100),
                'text': valid.string,
            },
            data
        )
    except api.ValueIssue as exc:
        print str(exc)

The above validates many things:

- that data has only 3 keys in the top-level dictionary
- that the values of each of those keys comply with their given validator

The clean dictionary (because you expected a dictionary) is then clean and ready
to be used by your program safely.

A ValueIssue will be raised with the ValueError message of each offending value
in data. (it does not stop at the first offending value)
'''

from . import util
from .exceptions import ExpectedIssue, KrankshaftError, ValueIssue
from django.core import validators
from django.core.exceptions import ValidationError
from django.core.files import File
from django.db import models
from django.template.defaultfilters import slugify
from django.utils import dateparse

class ExpecterHelper(object):
    '''
    This helper encapsulates the ability to recursively call expect from within
    a validator.

    Meant to be used exactly like the except function but allows access to
    the entire expecter.

        expect = ExpecterHelper(self, opts)
        expect(expected, data)
    '''
    def __init__(self, expecter, opts):
        self.expecter = expecter
        self.opts = opts

    def __call__(self, expected, data):
        return self.expect(expected, data, **self.opts)

    def __getattr__(self, name):
        return getattr(self.expecter, name)

class Expecter(object):
    '''
    The expect() runtime and handling of specific data structures.

    An Expecter simply contains some configuration options along with methods
    to handle specific data structures (dict, list).  This can be expanded or
    changed via subclasses which is about the only reason this exists
    as a class vs a function.

    Some specific details about how this implementation works.  The expect()
    function is the standard entry point for general users.

        from krankshaft import valid
        expecter = valid.Expecter()
        expecter.expect(valid.int, '1') # returns integer 1

    The expect function works recursively to dive through data structures so
    that the simple validators can be used within them and offer a way to
    define the expected data structure exactly.

    For dictionaries:

        expecter.expect({'key': valid.int}, {'key': '1'}) # returns {'key': 1}

    As you can see, the same valid.int validator is being used, but this time
    within a dict data structure.  The expect routine handles anything not
    callable (`hasattr(obj, '__call__')`) as a data structure that needs to
    be handled.  In the dict case, it iterates over the keys shared between
    the expected dict and the given data dict and applies the expected dict
    value (the validator) to the data dict value.  If a failure occurs for
    a specific key, the routine keeps track of the error but continues on to
    the next key.  In this way, all errors will be discovered versus only the
    first error.

    When errors are found, a `expecter.ValueIssue` is raised with all errors
    encountered.

    The expect dictionary logic is by default very strict about what keys exist
    in the given data.  By default, the keys in the given data must match
    exactly those in the expected dictionary.  You can lift those restrictions
    in a few ways.

        expecter = valid.Expecter(strict_dict=False)
        expecter = valid.Expecter(ignore_extra_keys=True)
        expecter = valid.Expecter(ignore_missing_keys=True)

        expecter.expect({...}, {...}, strict_dict=False)
        expecter.expect({...}, {...}, ignore_extra_keys=True)
        expecter.expect({...}, {...}, ignore_missing_keys=True)

    Either you configure the `Expecter` to handle dictionaries that why be
    default, or you override the option at the call site.  The `strict_dict`
    option is a quick way to make both extra/missing keys True or False.  So
    `strict_dict=False` would make `ignore_extra_keys` and `ignore_missing_keys`
    both True.

    Lists get some special non-intuitive (at first) treatment compared to
    dictionaries.  Let's start with the most obvious case:

        expecter.expect([valid.int, valid.int], ['1', 2]) # returns [1, 2]

    Makes sense right?  `expect()` iterates over the list and applies the
    validator to each matching member of the given data list.  And as you'd
    expect, if the given data list had more or less members than the expected
    list, it will blow up.

    Another list case, but a little more confusing:

        expecter.expect([], [1, 'a', None]) # returns [1, 'a', None]

    An empty list accepts all lists, since no validator is given.  An empty
    list is accepted as well.

    The last list case which may be the most confusing at first is this:

        expecter.expect([valid.int], [1, '2']) # returns [1, 2]

    A single validator in a list is special in that that validator is applied
    to all given members of the given data list.  It accepts zero or more
    values in the given data list.  It makes it easy to validate a list of
    homegenus elements which is very typical.

    This makes it a bit tough to validate a list of one or more members.  But
    there is a special validator for that.  The `list_n_or_more` is the
    validator you want:

        expecter.expect(valid.list_n_or_more(valid.int, 1), ['1', 2])

    The above will guarantee at least one member exists and that all members
    are `valid.int`s.

    As for tuple's, they behave just like lists.
    '''
    ExpectedIssue = ExpectedIssue
    ExpecterHelper = ExpecterHelper
    ValueIssue = ValueIssue

    defaults = {
        'ignore_extra_keys': False,
        'ignore_missing_keys': False,
    }

    field_to_validator = {} # setup at end of module

    def __init__(self, **opts):
        self.field_to_validator = self.field_to_validator.copy()
        self.opts = self.options(opts, self.defaults)

    def expect(self, expected, data, **opts):
        '''expect({'key': valid.int}, {'key': '1'}) -> {'key': 1}

        Clean and validate a given data structure using an expected structure.

        You may pass valid options directly here via opts.
        '''
        opts = self.options(opts, self.opts)

        if hasattr(expected, '__call__'):
            try:
                data = expected(data, self.ExpecterHelper(self, opts))

            except ValueError as exc:
                raise self.ValueIssue([self.format_exc(exc)])

            else:
                return data

        if expected.__class__ is not data.__class__:
            raise self.ValueIssue([
                'Unexpected type, expected %s: %s'
                % (expected.__class__, data.__class__)
            ])

        method = getattr(self, 'expect_' + expected.__class__.__name__, None)
        if not method:
            raise self.ExpectedIssue(
                'Your expected data structure is unhandled for type: %s'
                % expected.__class__
            )

        return method(expected, data, opts=opts)

    def expect_dict(self, expected, data, opts):
        '''
        Dictionary data structure handling with expect().

        You probably dont want to use this directly.
        '''
        clean = {}
        data_keys = set(data.keys())
        errors = {}
        expected_keys = set(expected.keys())

        if (
            not (opts['ignore_extra_keys'] and opts['ignore_missing_keys'])
            and expected_keys != data_keys
        ):
            extra_keys = data_keys - expected_keys
            missing_keys = expected_keys - data_keys
            if not opts['ignore_extra_keys'] and extra_keys:
                errors.setdefault('__nonkeyerrors__', []) \
                    .append('Extra keys: %s' % ', '.join(list(extra_keys)))

            if not opts['ignore_missing_keys'] and missing_keys:
                errors.setdefault('__nonkeyerrors__', []) \
                    .append('Missing keys: %s' % ', '.join(list(missing_keys)))

        for key in (expected_keys & data_keys):
            try:
                clean[key] = self.expect(expected[key], data[key], **opts)

            except self.ValueIssue as exc:
                errors[key] = exc.errors

        if errors:
            raise self.ValueIssue(errors)

        return clean

    def expect_list(self, expected, data, opts):
        '''
        List data structure handling with expect().

        You probably dont want to use this directly.
        '''
        clean = []
        errors = {}

        if len(expected) == 0:
            return data[:]

        elif len(expected) == 1:
            for i, value in enumerate(data):
                try:
                    clean.append(self.expect(expected[0], value, **opts))
                except self.ValueIssue as exc:
                    errors[i] = exc.errors

        elif len(expected) == len(data):
            for i, (cleaner, d) in enumerate(zip(expected, data)):
                try:
                    clean.append(self.expect(cleaner, d, **opts))
                except self.ValueIssue as exc:
                    errors[i] = exc.errors

        else:
            errors['__nonindexerrors__'] = [
                'Expected list of length %s, saw %s' % (
                    len(expected),
                    len(data),
                )
            ]

        if errors:
            raise self.ValueIssue(errors)

        return clean

    def expect_tuple(self, expected, data, opts):
        '''
        Tuple data structure handling with expect().

        You probably dont want to use this directly.
        '''
        return tuple(self.expect_list(expected, data, opts))

    def format_exc(self, exc):
        '''format_exc(exception) -> '...'

        Format an exception to be encoded into the error data structure.
        '''
        return str(exc)

    def from_field(self, field):
        '''from_field(model.field.field) -> validator or expected data structure

        Return a validator or an expected data structure (passable to expect())
        given a django model field.
        '''
        if field.__class__ in (
            models.ForeignKey,
            models.OneToOneField,
            models.ManyToManyField
        ):
            other_field = field
            while hasattr(other_field, 'rel') and other_field.rel:
                other_field = other_field.rel.get_related_field()

            validator = self.field_to_validator[other_field.__class__]

        else:
            try:
                validator = self.field_to_validator[field.__class__]
            except KeyError:
                raise self.ExpectedIssue(
                    'Unable to resolve "%s" to a validator, register your model'
                    ' field with api.expecter.register(Field, valid.validator)'
                    % str(field)
                )

        if not field.null:
            validator = no_none(validator)

        if field.choices:
            validator = choice(validator, tuple([
                value
                for value, display in field.choices
            ]))

        if hasattr(field, 'max_length') and field.max_length:
            validator = max_length(validator, field.max_length)

        if field.validators:
            validator = django_validator(validator, *field.validators)

        if field.__class__ is models.ManyToManyField:
            if field.blank:
                return [validator]

            else:
                return list_n_or_more(validator, 1)

        return validator

    def options(self, opts, defaults):
        '''
        Validate and set default options.
        '''
        return util.valid(
            util.defaults(self.shortcuts(opts), defaults),
            self.defaults.keys()
        )

    def register(self, field, validator):
        '''register(Field, valid.validator)

        Register a validator to be used for a given Field.
        '''
        self.field_to_validator[field] = validator

    def shortcuts(self, opts):
        '''
        Handle option shortcuts like `strict_dict`.
        '''
        if 'strict_dict' in opts:
            strict_dict = opts.pop('strict_dict')
            opts['ignore_extra_keys'] = not strict_dict
            opts['ignore_missing_keys'] = not strict_dict
        return opts

#
# validator helpers
#

def no_none(validator):
    '''or_none(__builtins__['int']) -> int_no_none_validator

    Given a function that returns ValueError when the given value is invalid,
    wrap it in such a way that it will properly handle being given None and
    raise ValueError.
    '''
    def no_none_validator(value, expect):
        if value is None:
            raise ValueError('%s does not accept None' % validator.__name__)
        return validator(value, expect)
    return wraps(no_none_validator, validator, '_no_none')

def or_none(validator):
    '''or_none(__builtins__['int']) -> int_or_none_validator

    Given a function that returns ValueError when the given value is invalid,
    wrap it in such a way that it will properly handle being given None and
    return None.
    '''
    def or_none_validator(value, expect):
        return None if value is None else validator(value, expect)
    return wraps(or_none_validator, validator, '_or_none')

def primitive(validator):
    '''primitive(int) -> int

    Upgrade a primitive validator to a validator that accepts the expect
    argument.
    '''
    def primitive_validator(value, expect):
        return validator(value)

    return wraps(primitive_validator, validator)

def wraps(wrapper, wrapped, suffix='', prefix=''):
    '''wraps(wrapper, validator)

    Wrap a validator in another validator.
    '''
    wrapper.__name__ = prefix + wrapped.__name__ + suffix

    return wrapper

#
# validator factories
#

def choice(validator, choices):
    '''choice(valid.string, ['a', 'b', 'c']) -> string_choices

    Wrap validator that also validates the returned value is in the list of
    valid choices.
    '''
    def choice_validator(value, expect):
        value = validator(value, expect)
        if value is not None and value not in choices:
            raise ValueError('The value is not a valid choice: %s' % value)
        return value

    return wraps(choice_validator, validator, '_choices')

def csv(validator, separator=','):
    '''csv(valid.int) -> int_csv

    Wrap validator so that it validates a list separated by commas.
    '''
    def csv_validator(data, expect):
        data = string_or_none(data, expect)

        if data is not None:
            clean = expect(
                [validator],
                [member for member in data.split(',') if member],
            )
            return ','.join([
                str(member)
                for member in clean
            ])

        return data

    return wraps(csv_validator, validator, '_csv')

def django_validator(validator, *validators):
    '''django_validator(valid.string, *field.validators) -> django_validator

    Use django validators as extra validation to a value.
    '''
    def django_validator_validator(value, expect):
        value = validator(value, expect)

        if value is not None:
            for django_is_valid in validators:
                try:
                    django_is_valid(value)
                except ValidationError as exc:
                    raise ValueError(exc.messages[0])

        return value

    return wraps(django_validator_validator, validator, '_django_validator')

def list_n_or_more(validator, n):
    '''list_n_or_more(valid.int, 1) -> list_1_or_more_int

    Wrap a validator that also validates the returned list has one or more
    members.
    '''
    if n < 1:
        raise KrankshaftError(
            'list_n_or_more only accepts values >= 1, not %s' % n
        )

    def list_n_or_more_validator(data, expect):
        clean = None
        errors = {}
        try:
            clean = expect([validator], data)
        except expect.ValueIssue as exc:
            errors = exc.errors

        if clean is not None and len(clean) < n:
            errors.setdefault('__nonindexerrors__', []) \
                .append(
                    'Expected list with %s or more elements, saw %s'
                    % (n, len(data))
                )

        if errors:
            raise expect.ValueIssue(errors)

        return clean

    return wraps(list_n_or_more_validator, validator,
        prefix='list_%s_or_more_' % n
    )

def max_length(validator, n):
    '''max_length(valid.string, 20) -> string_max_length_20

    Wrap a validator that also validates the returned value is less than the
    given max length.
    '''
    def max_length_validator(value, expect):
        value = validator(value, expect)
        if value is not None and len(value) > n:
            raise ValueError(
                'The value is greater than max length %s: %s' % (n, len(value))
            )
        return value

    return wraps(max_length_validator, validator, '_max_length_%s' % n)

def range(validator, low, high):
    '''range(valid.int, 0, 10) -> int_range_0_to_10

    Wrap validator that also validates the returned value lies within a given
    range.

    You can create an unbounded range by using None as either bound.
    '''
    def range_validator(value, expect):
        value = validator(value, expect)
        if value is not None and not (
            low <= value <= high
            if low is not None and high is not None
            else low <= value if low is not None
            else value <= high
        ):
            raise ValueError(
                'The value is not within the range %s <= %s <= %s'
                % (low, value, high)
            )
        return value

    return wraps(range_validator, validator, '_range_%s_to_%s' % (low, high))

float_range = lambda low, high: range(float, low, high)
float_or_none_range = lambda low, high: range(float_or_none, low, high)

int_range = lambda low, high: range(int, low, high)
int_or_none_range = lambda low, high: range(int_or_none, low, high)

string_max_length = lambda n: max_length(string, n)
string_or_none_max_length = lambda n: max_length(string_or_none, n)

#
# primitive validators
#

float = no_none(primitive(__builtins__['float']))
float_or_none = or_none(primitive(__builtins__['float']))

int = no_none(primitive(__builtins__['int']))
int_csv = no_none(csv(int))
int_or_none = or_none(primitive(__builtins__['int']))
int_csv_or_none = csv(int)

def string_or_none(value, expect):
    if value is not None:
        if not isinstance(value, basestring):
            raise ValueError('Expected string, saw: %s' % type(value))

    return value
string = no_none(string_or_none)

#
# complex validators
#

def bool_or_none(value, expect):
    '''bool_or_none('yes') -> True

    Take a truthy/falsey string and make it a boolean.
    '''
    truthy = ('1', 'true', 'yes')
    def str_or_none_lower(value, expect):
        return str(value).lower() if value is not None else None
    value = choice(
        str_or_none_lower,
        ('0', 'false', 'no', 'null') + truthy
    )(value, expect)

    if value is None:
        return None
    else:
        return value in truthy
bool = no_none(bool_or_none)

def date_or_none(value, expect):
    value = string_or_none(value, expect)

    if value is not None:
        clean = dateparse.parse_date(value)
        if clean is None:
            raise ValueError('Could not parse date: %s' % value)
        value = clean

    return value
date = no_none(date_or_none)

def datetime_or_none(value, expect):
    value = string_or_none(value, expect)

    if value is not None:
        clean = dateparse.parse_datetime(value)
        if clean is None:
            raise ValueError('Could not parse datetime: %s' % value)
        value = clean

    return value
datetime = no_none(datetime_or_none)

def django_file_or_none(value, expect):
    if value is not None and not isinstance(value, File):
        raise ValueError('No django file found')
    return value
django_file = no_none(django_file_or_none)

def django_image_or_none(value, expect):
    value = django_file_or_none(value, expect)

    if value is not None:
        try:
            from PIL import Image
            image = Image.open(value)
            image.load()

            value.seek(0)

            image = Image.open(value)
            image.verify()

            value.seek(0)

        except Exception:
            raise ValueError('No valid django image (file) found')

    return value
django_image = no_none(django_image_or_none)

email = django_validator(string, validators.validate_email)
email_or_none = django_validator(string_or_none, validators.validate_email)

def slug_or_none(value, expect):
    value = string_or_none(value, expect)
    if value is not None:
        value = slugify(value)
    return value
slug = no_none(slug_or_none)

def time_or_none(value, expect):
    value = string_or_none(value, expect)

    if value is not None:
        clean = dateparse.parse_time(value)
        if clean is None:
            raise ValueError('Could not parse time: %s' % value)
        value = clean

    return value
time = no_none(time_or_none)


Expecter.field_to_validator = {
    models.AutoField                    : int_or_none_range(1, 2147483647),
    models.BigIntegerField              : int_or_none_range(-9223372036854775808, 9223372036854775807),
    models.BooleanField                 : bool_or_none,
    models.CharField                    : string_or_none,
    models.CommaSeparatedIntegerField   : int_csv_or_none,
    models.DateField                    : date_or_none,
    models.DateTimeField                : datetime_or_none,
    models.DecimalField                 : string_or_none,
    models.EmailField                   : email_or_none,
    models.FileField                    : django_file_or_none,
    models.FilePathField                : string_or_none,
    models.FloatField                   : float_or_none,
    models.GenericIPAddressField        : string_or_none,
    models.IPAddressField               : string_or_none,
    models.ImageField                   : django_image_or_none,
    models.IntegerField                 : int_or_none_range(-2147483648, 2147483647),
    models.NullBooleanField             : bool_or_none,
    models.PositiveIntegerField         : int_or_none_range(0, 2147483647),
    models.PositiveSmallIntegerField    : int_or_none_range(0, 32767),
    models.SlugField                    : slug_or_none,
    models.SmallIntegerField            : int_or_none_range(-32768, 32767),
    models.TextField                    : string_or_none,
    models.TimeField                    : time_or_none,
    models.URLField                     : string_or_none,
}



from . import util
from .exceptions import KrankshaftError
from .query import DjangoQuery
from django.core.urlresolvers import reverse
from django.db import models

try:
    from django.db.transaction import atomic
except ImportError:
    from .compat.xact import xact as atomic

# TODO serialize file fields...
#   field: 'upload_to/file...',
#   field_href: '/media/upload_to/file...',
#
#   how to update that field?
#   should we strip upload_to?
#   we don't want to expose the upload_to path because that's a server side
#   setting... right?
#   maybe it doesnt matter...
#   how do we handle upload? just ignore the field?
class DjangoModelResource(object):
    '''
    Expose a Django Model as a RESTful resource.  Default implementation
    attempts to expose as much configurability throughout your app.  If you so
    choose, you can re-implement parts of this to trade flexibility for
    performance (ie: use `queryset.values(...)`).

    It's as simple as:

        from django.db.models import get_model
        from krankshaft.resource import DjangoModelResource

        @api
        class Object(DjangoModelResource):
            model = get_model('app', 'Object')

    Options:

        allowed_methods list of allowed HTTP methods for all endpoints
        allowed_methods_endpoint
                        list of allowed HTTP methods by endpoint
        excludes        list/tuple of field names to exclude from
                        serialize/deserialize
        fields          list/tuple of field names to serialize/deserialize
        use_location    instead of returning a serialized payload on POST/PUT,
                        use a Location header to point to where you may get
                        the representation (default: False)
        version_field   field name to use as Optimistic Concurrency Control
                        http://en.wikipedia.org/wiki/Optimistic_concurrency_control
                        This field becomes required in all requests to update
                        objects as it's tests for equality in the request
                        compared to the database.  It's a way to ensure that
                        if someone changes an object, it's changes are not
                        accidently overwritten by another requester.  No server
                        side locking required.
                        caveat: disables PUT to list endpoint as there is
                        no good way to handle this case (can be overridden by
                        version_field_allow_put_list = True if you want to
                        implement your own)
        version_field_allow_put_list
                        see version_field caveat

    '''
    Error = KrankshaftError
    Query = DjangoQuery
    allowed_methods = None
    allowed_methods_endpoint = None
    excludes = None
    fields = None
    model = None
    use_location = False
    version_field = None
    version_field_allow_put_list = None

    def __init__(self):
        self.fields = tuple([
            field
            for field in
                list(self.model._meta.fields)
                + [field for field, model in self.model._meta.get_m2m_with_model()]
            if self.fields is None or field.name in self.fields
        ])

        if self.excludes:
            self.fields = tuple([
                field
                for field in self.fields
                if field.name not in self.excludes
            ])

        if hasattr(self, 'exclude'):
            raise self.Error('You probably meant to use "excludes"')

        self.expected = {}
        self.expected_pk = None
        self.loaded = False
        self.related_lookup_cache = {}

        if self.allowed_methods:
            self.allowed_methods = tuple([
                method.lower()
                for method in self.allowed_methods
            ])

        if self.allowed_methods_endpoint:
            self.allowed_methods_endpoint = {
                endpoint: tuple([
                    method.lower()
                    for method in methods
                ])
                for endpoint, methods in
                    self.allowed_methods_endpoint.iteritems()
                if methods
            }

        if self.version_field \
           and self.version_field_allow_put_list is None:
            self.version_field_allow_put_list = False

    def allowed(self, endpoint, map):
        '''allowed('list', {'get': self.get, ...})

        Filter the given route method map based on specified allowed methods.
        '''
        tocheck = [
            self.allowed_methods,
            self.allowed_methods_endpoint.get(endpoint)
                if self.allowed_methods_endpoint else None,
        ]
        newmap = {}
        for method, view in map.iteritems():
            allowed = True
            for methods in tocheck:
                if methods and method.lower() not in methods:
                    allowed = False
                    break

            if allowed:
                newmap[method] = view

        return newmap

    def clean(self, request, data, list=False):
        '''clean(request, data) -> clean

        Clean data from user environment suitable for applying to an instance
        or creating an instance.
        '''
        expected = self.expected
        if list:
            expected = [expected]

        try:
            return self.api.expect(expected, data, strict_dict=False)
        except self.api.ValueIssue as exc:
            self.api.abort(self.api.serialize(request, 422, {
                'error': 'Supplied data was invalid',
                'invalid': exc.errors,
            }))

    def clean_id(self, request, id):
        '''clean_id(request, id) -> clean_id

        Useful in situations where the id may be in an improper format.

        For example:

            ids = [self.clean_id(request, id) for id in '1;2;3;4'.split(';')]

        Without `clean_id()`, the id would be a string.  So when we go to check
        to see if we fetched all the proper instances from the database, a check
        like:

            set(ids) == set([instance.pk for instance in instances])

        Would fail simply because '1' (string) != 1 (integer).

        The request argument can be given None, but in this case, you are
        responsible for handling any raised api.ValueIssue.  If a request is
        given, this will automatically call `abort()` with an error response.
        '''
        try:
            return self.api.expect(self.expected_pk, id)
        except self.api.ValueIssue as exc:
            if not request:
                raise

            self.api.abort(self.api.serialize(request, 400, {
                'error': 'Invalid ID for model %s' % self.model.__name__,
                'invalid': exc.errors,
            }))

    def check(self, request, instance, clean, abortable=True):
        '''check(request, instance, clean)

        Aborts if any checks given an instance and the clean()ed data fail.

        This is where version_field is used to test if the database and request
        are in sync.

        If abortable is False, the response code and error detail that would
        have been serialized with the response is returned.

            code, detail = \\
                self.check(request, instance, clean, abortable=False)

            if code:
                if detail:
                    self.api.abort(self.api.serialize(request, code, detail))

                else:
                    self.api.abort(request, code)

        '''
        # effectively use "goto" conventions, not really looping here
        code = None
        detail = None
        while True:
            if self.version_field and instance:
                if self.version_field not in clean:
                    code = 400
                    detail = {
                        'error': 'The "%s" field must be specified' \
                        % self.version_field
                    }
                    break

                version_db = getattr(instance, self.version_field)
                version_clean = clean[self.version_field]
                if version_db != version_clean:
                    code = 409
                    break

            break

        if abortable and code:
            if detail:
                self.api.abort(self.api.serialize(request, code, detail))
            else:
                self.api.abort(request, code)

        return code, detail

    def deserialize(self, request, data, instance=None):
        '''deserialize(request, data, instance=instance) -> instance

        Convenience method for clean(), check(), and update().
        '''
        if instance is None:
            instance = self.model()

        clean = self.clean(request, data)
        self.check(request, instance, clean)
        return self.update(instance, clean)

    def endpoint(self, name):
        '''endpoint('list') -> 'api_v1_resource_list'

        Construct a name to be used in Django's reverse() or url(..., name=...).
        '''
        return self.api.endpoint(self.name + '_' + name)

    def fetch(self, *ids):
        '''fetch(*ids) -> [instance, ...]

        Fetch given resource primary keys from the database.  This enables
        resources to request other resources to fetch their related instances.

        When a resource attempts to resolve a URI into an instance:

            resource, ids = api.resolve(['/path/to/resource/1/'])
            instances = resource.fetch(*ids)

        It's convention to use the `fetch()` on a resource to retrieve the
        corresponding instances from the database.

        Order of requested ids is preserved.

        Allows `clean_id()` to raise Exceptions if given ids are invalid.  You
        are responsible for handling api.ValueIssue being raised from this.

        Model.DoesNotExist is raised when a given id(s) does not exist in the
        database.
        '''

        ids = [self.clean_id(None, id) for id in ids]
        instances = list(self.get_query_set(None).filter(pk__in=ids))
        set_ids = set(ids)
        set_instance_pks = set([instance.pk for instance in instances])
        if set_ids != set_instance_pks:
            missing = set_ids - set_instance_pks
            raise self.model.DoesNotExist(
                'Could not find instances for: %s' % ', '.join([
                    str(id) for id in list(missing)
                ])
            )

        instances.sort(key=lambda instance: ids.index(instance.pk))
        return instances

    def fetch_list(self, request, query):
        '''fetch_list(request, query) -> instances

        Ideally, calling functions would do:

            query, data = self.api.deserialize(request)
            instances, meta = self.fetch_list(request, query)

        '''
        if not isinstance(query, self.Query):
            query = self.Query(query)
        qs, meta = query.apply(self.get_query_set(request))
        instances = list(qs)

        if meta['next']:
            meta['next'] = self.reverse('list') + meta['next']

        if meta['previous']:
            meta['previous'] = self.reverse('list') + meta['previous']

        for instance in instances:
            self.is_authorized(request, instance)

        return instances, meta

    def fetch_set(self, request, idset):
        '''fetch_set(request, idset) -> instances

        Fetch a set of instances given idset which is in the format of a string
        of semi-colon separated ids.  Handles ensuring all instances are found.
        '''
        idset = [self.clean_id(request, id) for id in idset.split(';')]
        instances = list(self.get_query_set(request).filter(pk__in=idset))

        set_ids = set(idset)
        set_instance_pks = set([instance.pk for instance in instances])
        if set_ids != set_instance_pks:
            self.api.abort(self.api.serialize(request, 404, {
                'error': 'Missing some requested objects',
                'missing': list(set_ids - set_instance_pks),
            }))

        for instance in instances:
            self.is_authorized(request, instance)

        return instances

    def fetch_single(self, request, query, id):
        '''fetch_single(request, query, id) -> instance

        Fetch a single instance from the database handling when an instance
        isn't found or multiple are found when only one was expected.
        '''
        if query and not isinstance(query, self.Query):
            query = self.Query(query)

        qs = self.get_query_set(request)
        meta = None
        if query:
            qs, meta = query.apply(qs)

        try:
            instance = qs.get(pk=id)

        except self.model.DoesNotExist:
            self.api.abort(self.api.response(request, 404))

        self.is_authorized(request, instance)

        return instance

    def get_query_set(self, request):
        '''get_query_set(request) -> queryset

        Request may be `None` if no request authorization need be done.

        Typically, an endpoint will pass in the current request to possibly
        limit the queryset based on how the authorization of the API is setup.

        This also automatically sets up prefetching of ManyToManyField's.
        '''
        qs = self.model.objects.all()

        for field in self.fields:
            if isinstance(field, models.ManyToManyField):
                qs = qs.prefetch_related(field.name)

        if request is None:
            return qs
        else:
            return request.auth.limit(qs)

    def is_authorized(self, request, instance):
        '''is_authorized(request, instance)

        Abort if the request is not authorized for the instance.
        '''
        if not request.auth.is_authorized_object(instance):
            self.api.abort(self.api.challenge(request, request.auth))

    def load(self):
        '''load()

        Called once all resources are registered allowing resources to auto
        configure themselves with eachother because they're all guaranteed to
        be registered at this point.

        This is typically invoked automatically once the API its registered with
        has its `.urls` property accessed.
        '''
        if self.loaded:
            return

        # preload related resources
        for field in self.fields:
            if not isinstance(field, (models.ForeignKey, models.ManyToManyField)):
                continue

            self.related_lookup_cache[field] = self.related_lookup(field)

        self.loaded = True
        self.load_post()

    def load_post(self):
        '''load_post()

        Routines to be run after load() because they depend on what happens
        during load() and depend on a post load state.
        '''
        self.expected.update({
            field.name: self.api.expecter.from_field(field)
            for field in self.fields
        })
        for field in self.fields:
            is_many = isinstance(field, models.ManyToManyField)
            is_related = is_many or isinstance(field, models.ForeignKey)
            if not is_related:
                continue

            self.expected[field.name] = self.make_related_validator(
                field,
                self.expected[field.name],
                many=is_many
            )

        self.expected_pk = self.expected[self.pk_name]

    def make_related_validator(self, field, expected, many=False):
        def related_validator(value, expect):
            resource = self.related_lookup(field)
            if value is not None and resource:
                try:
                    resolved_resource, value = \
                        self.api.resolve(value if many else [value])

                    if resolved_resource != resource:
                        raise self.api.ResolveError(
                            'Unexpected resource found: %s (expected %s)'
                            % (resolved_resource.name, resource.name)
                        )

                except self.api.ResolveError as e:
                    raise ValueError(
                        'Unable to resolve related: %s, %s' % (value, str(e))
                    )

                if not many:
                    value = value[0]

            return expect(expected, value)

        return related_validator

    def make_view(self, name, methods):
        '''make_view(self.endpoint('list'), {...}) -> resource_view

        Make a view that can resolve back to this resource and is suitable for
        use.
        '''
        def view(request, *args, **kwargs):
            return self.api.route(methods, request, args, kwargs)

        view.__name__ = name

        # so resolve() can find its way back from the view to the resource
        view.resource = self

        return view

    @property
    def name(self):
        '''
        Return a default resource name based on the resources class name.
        Removes instances of "Resource" in the name.

        Override by simply assigning a name in the class definition.
        '''
        return self.__class__.__name__.lower().replace('resource', '')

    @property
    def pk_name(self):
        return self.model._meta.pk.name

    def related(self, field, id):
        '''related(field, 1) -> 1 or '/path/to/resource/1/'

        Handle serializing related fields.  Default implementation is to
        return a URI representing the path to the resource.
        '''
        resource = self.related_lookup(field)
        if resource and id is not None:
            return resource.reverse_single(id)
        else:
            return id

    def related_lookup(self, field):
        '''related_lookup() -> resource

        Lookup a resource for a related field.

        Returns None if a resource cannot be found.
        '''
        if self.loaded:
            return self.related_lookup_cache[field]

        else:
            model = field.rel.to
            for resource in self.api.registered_resources:
                if model is getattr(resource, 'model', None):
                    if hasattr(resource, 'instance'):
                        # adjust for API Class Helper
                        resource = resource.instance
                    return resource
            return None

    def reverse(self, name, *args, **kwargs):
        '''reverse('list') -> '/api/v1/resource/'

        Shortcut for creating urls related to this resource.
        '''
        return reverse(self.endpoint(name), *args, **kwargs)

    def reverse_single(self, id):
        '''reverse_single(instance.pk) -> '/resource/1/'

        Shortcut for creating a resource_uri.
        '''
        return self.reverse('single', args=(id,))

    @property
    def routes(self):
        '''
        Return a map of HTTP Method routes under an endpoint.

        ie:
            prefix = r'^%s/' % self.name
            return (
                ('name_of_url', prefix + r'endpoint/$', {
                    'method': self.view,
                    ...
                }),
                ...
            )

        '''
        prefix = r'^%s/' % self.name
        list_methods = {
            'delete': self.delete_list,
            'get': self.get_list,
            'post': self.post_list,
            'put': self.put_list,
        }
        if self.version_field and not self.version_field_allow_put_list:
            del list_methods['put']

        return (
            (self.endpoint('list'), prefix + '$',
                self.allowed('list', list_methods)
            ),
            (self.endpoint('single'), prefix + '(?P<id>[^/]+)/$',
                self.allowed('single', {
                    'delete': self.delete,
                    'get': self.get,
                    'put': self.put,
                })
            ),
            (self.endpoint('set'), prefix + 'set/(?P<idset>[^/](?:[^/]|;)*)/$',
                self.allowed('set', {
                    'delete': self.delete_set,
                    'get': self.get_set,
                    'put': self.put_set,
                })
            ),
        )

    def serialize(self, instance):
        '''serialize(instance) -> {...}

        Returns a dictionary ready for serialization into a response.  Suitable
        for using `api.serialize(request, code, ...)`.
        '''
        from django.db.models.query_utils import DeferredAttribute

        data = {}
        for field in self.fields:
            if (
                instance._deferred \
                and isinstance(
                    instance.__class__.__dict__.get(field.attname),
                    DeferredAttribute
                )
            ):
                # skip deferred fields
                continue

            method = getattr(self, 'serialize_%s' % field.name, None)
            if method:
                data[field.name] = method(instance, field)

            else:
                if isinstance(field, models.ManyToManyField):
                    qs = getattr(instance, field.name).all()
                    data[field.name] = [
                        self.related(field, related.pk)
                        for related in qs
                    ]
                    data[field.name + '_id'] = [
                        related.pk
                        for related in qs
                    ]

                elif isinstance(field, models.ForeignKey):
                    data[field.name] = self.related(
                        field,
                        getattr(instance, field.name + '_id')
                    )
                    data[field.name + '_id'] = \
                        getattr(instance, field.name + '_id')

                else:
                    data[field.name] = getattr(instance, field.name)

        data['resource_uri'] = self.reverse_single(instance.pk)

        return data

    def update(self, instance, clean):
        '''update(instance, clean) -> instance

        Update an instance with clean()ed data specified in the request.

        No further saving is required after this has been called.
        '''
        manytomany = {}
        for field in self.fields:
            if field.name not in clean:
                # since its not strictly enforcing extra/missing keys, if
                # the client didn't supply the field, stick with model defaults
                continue

            method = getattr(self, 'deserialize_%s' % field.name, None)
            if method:
                setattr(instance, field.name, method(instance, field, clean))

            else:
                if isinstance(field, models.ManyToManyField):
                    resource = self.related_lookup(field)
                    manytomany[field] = resource.fetch(*clean[field.name])

                elif isinstance(field, models.ForeignKey):
                    setattr(instance, field.name + '_id', clean[field.name])

                else:
                    setattr(instance, field.name, clean[field.name])

        instance.save()
        for field, instances in manytomany.iteritems():
            manager = getattr(instance, field.name)
            manager.clear()
            manager.add(*instances)

            if hasattr(instance, '_prefetched_objects_cache') \
               and field.name in instance._prefetched_objects_cache:
                instance._prefetched_objects_cache[field.name] = \
                    instances

        return instance

    @property
    def urls(self):
        '''
        Return the Django URLs needed to hook up this resource to URL routing.

        Typically you do not need to use this as the `api.urls` call will
        automatically pull this in.
        '''
        from django.conf.urls import patterns, include, url

        return patterns('', *[
            url(regex, self.api.wrap(self.make_view(name, methods)), name=name)
            for name, regex, methods in self.routes
        ])


    def delete(self, request, id):
        '''
        Deletes the object pointed to by id.

            /resource/1/

        '''
        with atomic():
            instance = self.fetch_single(request, None, id=id)
            instance.delete()

        return self.api.response(request, 204)

    def delete_list(self, request):
        '''
        Deletes the instances pointed to by the given query (specified by
        the query string).

            /resource/?field=value

        '''
        query, data = self.api.deserialize(request)

        with atomic():
            instances, meta = self.fetch_list(request, query)
            for instance in instances:
                instance.delete()

        return self.api.response(request, 204)

    def delete_set(self, request, idset):
        '''
        Delete the given set of objects (specified in the path as an id list
        separated by a semi-colon).

            /resource/set/1;2;3/

        '''
        with atomic():
            instances = self.fetch_set(request, idset)
            for instance in instances:
                instance.delete()

        return self.api.response(request, 204)


    def get(self, request, id):
        '''
        Get the serialized value of an object.

            /resource/1/

        '''
        query, data = self.api.deserialize(request)
        instance = self.fetch_single(request, query, id=id)

        return self.api.serialize(request, 200, self.serialize(instance))

    def get_list(self, request):
        '''
        Get the serialized value of a list of objects of a given query
        (specified as the query string).

            /resource/?field=value

        '''
        query, data = self.api.deserialize(request)
        instances, meta = self.fetch_list(request, query)

        return self.api.serialize(request, 200, {
            'meta': meta,
            'objects': [
                self.serialize(instance)
                for instance in instances
            ]
        })

    def get_set(self, request, idset):
        '''
        Get the serialized value of a set of objects given by a list of ids
        separated by a semi-colon.

            /resource/set/1;2;3/

        '''
        instances = self.fetch_set(request, idset)

        return self.api.serialize(request, 200, {
            'objects': [
                self.serialize(instance)
                for instance in instances
            ]
        })


    def post_list(self, request):
        '''
        Create a new object.

            /resource/

        '''
        query, data = self.api.deserialize(request)

        with atomic():
            instance = self.deserialize(request, data)

        if self.use_location:
            return self.api.response(request, 204,
                Location=self.reverse_single(instance.pk)
            )
        else:
            return self.api.serialize(request, 200, self.serialize(instance))


    def put(self, request, id):
        '''
        Update an object.

            /resource/1/

        '''
        query, data = self.api.deserialize(request)

        with atomic():
            instance = self.fetch_single(request, query, id=id)
            instance = self.deserialize(request, data, instance)

        if self.use_location:
            return self.api.response(request, 204,
                Location=self.reverse_single(instance.pk)
            )
        else:
            return self.api.serialize(request, 200, self.serialize(instance))

    def put_list(self, request):
        '''
        Given a single dictionary of fields and values, update all the instances
        with the given field/values.

        Ideally, a subset of all fields is given.  Enables mass update to
        an unknown set of objects (those matching a query).

            /resource/?field=value

        '''
        query, data = self.api.deserialize(request)

        # dont modify only part of the query, modify the whole thing
        query = self.Query(query).without('defer', 'limit', 'offset', 'only')

        with atomic():
            instances, meta = self.fetch_list(request, query)
            for instance in instances:
                self.deserialize(request, data, instance)

        if self.use_location:
            idset = ';'.join([
                str(instance.pk)
                for instance in instances
            ])
            return self.api.response(request, 204,
                Location=self.reverse('set', args=(idset, ))
            )

        else:
            return self.api.serialize(request, 200, {
                'meta': meta,
                'objects': [
                    self.serialize(instance)
                    for instance in instances
                ]
            })

    def put_set(self, request, idset):
        '''
        Update a set of objects.

            /resource/set/1;2;3/

        '''
        query, data = self.api.deserialize(request)

        with atomic():
            instances = self.fetch_set(request, idset)
            clean = self.clean(request, data, list=True)

            if not all(self.pk_name in obj for obj in clean):
                return self.api.serialize(request, 400, {
                    'error': 'You must supply the primary key with each object'
                })

            instance_lookup = {
                instance.pk: instance
                for instance in instances
            }

            code_and_details = []
            for obj in clean:
                id = obj[self.pk_name]
                instance = instance_lookup[id]
                code, detail = \
                    self.check(request, instance, obj, abortable=False)
                if code:
                    code_and_details.append((code, id, detail))

            if code_and_details:
                codes = list(set([
                    code for code, id, detail in code_and_details
                ]))
                has_details = any(
                    detail for code, id, detail in code_and_details
                )
                if len(codes) == 1:
                    if has_details:
                        self.api.abort(self.api.serialize(request, codes[0], {
                            'invalid': {
                                id: detail
                                for code, id, detail in code_and_details
                            }
                        }))

                    else:
                        self.api.abort(request, codes[0])
                else:
                    # TODO really not thrilled with this... how is the client
                    # supposed to handle this?
                    # TODO figure out all 4xx responses that can come out of the
                    # API and figure out a standardized way to handle the
                    # response
                    # at this point, at least they get _something_, but their
                    # only reasonable course of action is to bail out completely
                    # start documenting API 4xx errors page? and how to get
                    # each one?
                    # document them on a per method endpoint basis... seems like
                    # the __doc__ on the method makes a lot of sense...
                    self.api.abort(self.api.serialize(request, 400, {
                        'error': 'Mixed error codes',
                        'invalid': {
                            id: util.defaults({}, detail or {}, {
                                'code': code,
                            })
                            for code, id, detail in code_and_details
                        }
                    }))

            for obj in clean:
                id = obj[self.pk_name]
                instance = instance_lookup[id]
                self.deserialize(request, obj, instance)

        if self.use_location:
            return self.api.response(request, 204,
                Location=self.reverse('set', args=(idset,))
            )

        else:
            return self.api.serialize(request, 200, {
                'objects': [
                    self.serialize(instance)
                    for instance in instances
                ]
            })



from __future__ import absolute_import

from django.db import models
from krankshaft.exceptions import InvalidOptions
from krankshaft.query import DjangoQuery, Query
from tests.base import TestCaseNoDB
import pytest

class FakeForeign(models.Model):
    char_indexed = models.CharField(max_length=20, db_index=True)
    char_unindexed = models.CharField(max_length=20)

class Fake(models.Model):
    char_indexed = models.CharField(max_length=20, db_index=True)
    char_unindexed = models.CharField(max_length=20)
    foreign = models.ForeignKey(FakeForeign)

class QueryTest(TestCaseNoDB):
    def setUp(self):
        self.Query = Query

    def test_apply_not_implemented(self):
        self.assertRaises(NotImplementedError, self.Query({}).apply, None)

    def test_copy(self):
        a = self.Query({})
        b = a.copy()
        self.assertNotEqual(a, b)
        self.assertEqual(a.qs, b.qs)
        self.assertEqual(a.opts, b.opts)

    def test_default(self):
        query = self.Query({})
        self.assertEqual(query.opts, self.Query.defaults)

    def test_invalid_options(self):
        self.assertRaises(InvalidOptions, self.Query, {}, {
            'not_a_valid_option': True
        })

    def test_without(self):
        query = self.Query({'order_by': 'something'})
        without = query.without('order_by')
        qs = query.qs.copy()
        del qs['order_by']
        self.assertEqual(qs, without.qs)

class DjangoQueryTest(QueryTest):
    def apply_assert(self, qs, **opts):
        queryset, meta = self.Query(qs, opts).apply(Fake.objects.all())
        self.assertNotEqual(queryset, None)
        return queryset

    def apply_raises(self, qs, **opts):
        self.assertRaises(
            self.Query.Issues,
            self.Query(qs, opts).apply, Fake.objects.all()
        )

    def setUp(self):
        self.Query = DjangoQuery

    def test_apply_not_implemented(self):
        pass # its implemented on the DjangoQuery subclass

    def test_default_with_options(self):
        query = self.Query({}, {'default_limit': 20})
        opts = self.Query.defaults.copy()
        opts['default_limit'] = 20
        self.assertEqual(query.opts, opts)

    def test_apply_defer(self):
        queryset = self.apply_assert({'defer': 'char_unindexed,char_indexed'})
        assert queryset.query.deferred_loading[1]
        assert 'char_indexed' in queryset.query.deferred_loading[0]
        assert 'char_unindexed' in queryset.query.deferred_loading[0]

    def test_apply_defer_disallow_cross_relation(self):
        self.apply_raises({'defer': 'foreign__char_unindexed'})

    def test_apply_defer_disallow_cross_relation_override(self):
        self.assertRaises(
            self.Query.Issues,
            self.Query(
                {'defer': 'foreign__char_unindexed'},
                {'defer_allow_related': True}
            ).apply,
            Fake.objects.all(),
            defer_allow_related=False
        )

    def test_apply_defer_allow_cross_relation(self):
        queryset = self.apply_assert({'defer': 'foreign__char_unindexed'}, defer_allow_related=True)
        assert queryset.query.deferred_loading[1]
        assert 'foreign__char_unindexed' in queryset.query.deferred_loading[0]

    def test_apply_filter(self):
        self.apply_assert({'char_indexed': 'value'})

    def test_apply_filter_any(self):
        self.apply_assert({'char_indexed': 'value'}, filter=self.Query.Any)

    def test_apply_filter_specified(self):
        self.apply_assert({'char_indexed': 'value'}, filter=('char_indexed', ))

    def test_apply_filter_allow_cross_relation(self):
        self.apply_assert({'foreign__char_indexed': 'value'}, filter_allow_related=True)

    def test_apply_filter_disallow_cross_relation(self):
        self.apply_raises({'foreign__char_indexed': 'value'})

    def test_apply_filter_specified_but_not_used(self):
        self.apply_raises({'char_unindexed': 'value'}, filter=('char_indexed', ))

    def test_apply_filter_unindexed(self):
        self.apply_raises({'char_unindexed': 'value'})

    def test_apply_filter_at_least_one_indexed(self):
        self.apply_assert({'char_indexed': 'value', 'char_unindexed': 'value'})

    def test_apply_filter_lookup_any(self):
        self.apply_assert({'char_indexed__exact': 'value'}, filter_lookups=self.Query.Any)

    def test_apply_filter_lookup_specified(self):
        self.apply_assert({'char_indexed__exact': 'value'}, filter_lookups=('exact', ))

    def test_apply_filter_lookup_specified_but_not_used(self):
        self.apply_raises({'char_indexed__in': 'v1,v2'}, filter_lookups=('exact', ))

    def test_apply_filter_lookup_exact(self):
        self.apply_assert({'char_indexed__exact': 'v1'})

    def test_apply_filter_lookup_in(self):
        self.apply_assert({'char_indexed__in': 'v1,v2,v3'})

    def test_apply_filter_lookup_range(self):
        self.apply_assert({'char_indexed__range': 'v1,v2'})

    def test_apply_filter_lookup_range_more_than_2_params(self):
        self.apply_raises({'char_indexed__range': 'v1,v2,v3'})

    def test_apply_filter_lookup_isnull_truthy(self):
        self.apply_assert({'char_indexed__isnull': 'true'})

    def test_apply_filter_lookup_isnull_falsey(self):
        self.apply_assert({'char_indexed__isnull': 'false'})

    def test_apply_filter_lookup_isnull_invalid_param(self):
        self.apply_raises({'char_indexed__isnull': 'a'})

    def test_apply_limit(self):
        queryset = self.apply_assert({'limit': '1'})
        self.assertEqual(queryset.query.low_mark, 0)
        self.assertEqual(queryset.query.high_mark, 1)

    def test_apply_limit_default_limit(self):
        queryset = self.apply_assert({}, default_limit=1)
        self.assertEqual(queryset.query.low_mark, 0)
        self.assertEqual(queryset.query.high_mark, 1)

    def test_apply_limit_invalid(self):
        self.apply_raises({'limit': 'a'})

    def test_apply_limit_invalid_negative(self):
        self.apply_raises({'limit': '-1'})

    def test_apply_limit_gt_max(self):
        self.apply_raises({'limit': '21'}, max_limit=20)

    def test_apply_limit_is_zero(self):
        queryset = self.apply_assert({'limit': '0'})
        self.assertEqual(queryset.query.low_mark, 0)
        self.assertEqual(queryset.query.high_mark, None)

    def test_apply_limit_is_zero_with_max_limit(self):
        queryset = self.apply_assert({'limit': '0'}, max_limit=20)
        self.assertEqual(queryset.query.low_mark, 0)
        self.assertEqual(queryset.query.high_mark, 20)

    def test_apply_meta_page_0(self):
        queryset, meta = \
            self.Query({'limit': '10', 'char_indexed': 'value'}) \
            .apply(Fake.objects.all())

        assert meta['next'] == '?char_indexed=value&limit=10&offset=10'
        assert meta['previous'] is None

    def test_apply_meta_page_1(self):
        queryset, meta = \
            self.Query({'offset': '10', 'limit': '10', 'char_indexed': 'value'}) \
            .apply(Fake.objects.all())

        assert meta['next'] == '?char_indexed=value&limit=10&offset=20'
        assert meta['previous'] == '?char_indexed=value&limit=10&offset=0'

    def test_apply_offset(self):
        queryset = self.apply_assert({'limit': '1', 'offset': '1'})
        self.assertEqual(queryset.query.low_mark, 1)
        self.assertEqual(queryset.query.high_mark, 2)

    def test_apply_offset_invalid(self):
        self.apply_raises({'limit': '1', 'offset': 'a'})

    def test_apply_offset_invalid_negative(self):
        self.apply_raises({'limit': '1', 'offset': '-1'})

    def test_apply_offset_gt_max(self):
        self.apply_raises({'limit': '1', 'offset': '21'}, max_offset=20)

    def test_apply_only(self):
        queryset = self.apply_assert({'only': 'char_unindexed,char_indexed'})
        assert not queryset.query.deferred_loading[1]
        assert 'char_indexed' in queryset.query.deferred_loading[0]
        assert 'char_unindexed' in queryset.query.deferred_loading[0]

    def test_apply_only_disallow_cross_relation(self):
        self.apply_raises({'only': 'foreign__char_unindexed'})

    def test_apply_only_allow_cross_relation(self):
        queryset = self.apply_assert({'only': 'foreign__char_unindexed'}, only_allow_related=True)
        assert not queryset.query.deferred_loading[1]
        assert 'foreign__char_unindexed' in queryset.query.deferred_loading[0]

    def test_apply_order_by_indexed(self):
        self.apply_assert({'order_by': 'char_indexed'})

    def test_apply_order_by_at_least_one_indexed(self):
        self.apply_assert({'order_by': 'char_unindexed,char_indexed'})

    def test_apply_order_by_not_indexed(self):
        self.apply_raises({'order_by': 'char_unindexed'})

    def test_apply_order_by_disallow_cross_relation(self):
        self.apply_raises({'order_by': 'foreign__char_unindexed'})

    def test_apply_order_by_allow_cross_relation_indexed(self):
        self.apply_assert({'order_by': 'foreign__char_indexed'}, ordering_allow_related=True)

    def test_apply_order_by_allow_cross_relation_unindexed(self):
        self.apply_raises({'order_by': 'foreign__char_unindexed'}, ordering_allow_related=True)

    def test_apply_order_by_allow_specific_fields(self):
        self.apply_assert({'order_by': 'char_unindexed'}, ordering=('char_unindexed', ))

    def test_apply_order_by_allow_specific_fields_not_used(self):
        self.apply_raises({'order_by': 'char_indexed'}, ordering=('char_unindexed', ))

    def test_apply_order_by_allow_any_fields(self):
        self.apply_assert({'order_by': 'char_unindexed'}, ordering=self.Query.Any)

    def test_apply_order_by_allow_specific_fields_cross_relation_disallowed(self):
        self.apply_raises({'order_by': 'foreign__char_unindexed'}, ordering=('foreign__char_unindexed', ))

    def test_apply_order_by_allow_specific_fields_cross_relation_allowed(self):
        self.apply_assert({'order_by': 'foreign__char_unindexed'}, ordering_allow_related=True, ordering=('foreign__char_unindexed', ))

    def test_parse_field_multiple_lookups(self):
        self.assertRaises(self.Query.Error, self.Query({}).parse_field, 'field__in__in')



from .exceptions import KrankshaftError
from collections import namedtuple
from cStringIO import StringIO
from datetime import datetime, date, time, timedelta
from functools import partial
import decimal
import json
import mimeparse
import urlparse

# TODO: Accept: */* dumps: no method to_urlencoded
#       maybe only consider serializers we have a method for?
# TODO use 406 Not Acceptable in case where we cannot serialize

class Serializer(object):
    '''
    Create a serializer to register serializable's against.  Understands how to
    serialize objects to a data representation.
    '''
    class SerializableExists(KrankshaftError): pass
    class Unsupported(KrankshaftError): pass

    SerializedContainer = \
        namedtuple('SerializedContainer', ['data', 'content_type'])

    content_types = {
        'application/json': 'json',
        'application/x-www-form-urlencoded': 'urlencoded',
        'multipart/form-data': 'multipart_form_data',
    }
    default_content_type = 'application/json'
    format_to_content_type = {
        'json': 'application/json',
    }

    dict_like_iterables = (
        dict,
    )
    list_like_iterables = (
        list,
        tuple,
    )
    primitive_classes = (
        int,
        long,
        float,
        basestring,
    )
    primitive_values = (
        None,
        True,
        False,
    )

    def __init__(self, default_content_type=None):
        if default_content_type:
            self.default_content_type = default_content_type

        self.complex = (
            # date/time
            (date, self.convert_datetime_ish),
            (datetime, self.convert_datetime_ish),
            (time, self.convert_datetime_ish),
            (timedelta, self.convert_timedelta),

            # other
            (decimal.Decimal, self.convert_decimal),
        )

        self.registered = {}

    def convert(self, obj, **opts):
        '''convert(obj) -> serializable

        Convert an arbitrary object into something serializable.

        Raises TypeError if unable to convert.
        '''
        if obj in self.primitive_values:
            return obj

        elif isinstance(obj, self.primitive_classes):
            return obj

        elif isinstance(obj, self.list_like_iterables):
            return [
                self.convert(subobj, **opts)
                for subobj in obj
            ]

        elif isinstance(obj, self.dict_like_iterables):
            return {
                self.convert(key, **opts): self.convert(val, **opts)
                for key, val in obj.items()
            }

        for klass, method in self.complex:
            if isinstance(obj, klass):
                return method(obj)

        for klass in (obj.__class__, ) + getattr(obj, '__bases__', ()):
            if klass in self.registered:
                serializable = self.registered[klass](obj)
                return serializable.convert(**opts)

        return self.convert_unknown(obj)

    def convert_datetime_ish(self, obj):
        return obj.isoformat()

    def convert_decimal(self, obj):
        return str(obj)

    def convert_timedelta(self, obj):
        return obj.total_seconds()

    def convert_unknown(self, obj):
        raise TypeError(repr(obj) + ' is not serializable')

    def deserialize(self, body, content_type, **opts):
        '''deserialize(body, content_type) -> data

        Deserialize a request body into a data-structure.
        '''
        method = getattr(self, 'from_%s' % self.get_format(content_type)[1])
        return method(body, content_type=content_type, **opts)

    def deserialize_request(self, request, content_type, **opts):
        format = self.get_format(content_type)[1]
        if format == 'multipart_form_data':
            return self.from_multipart_form_data(request, **opts)
        else:
            return self.deserialize(request.body, content_type, **opts)

    def from_urlencoded(self, body, **opts):
        data = urlparse.parse_qs(
            body,
            keep_blank_values=True,
            strict_parsing=True
        )
        return {
            key: value[-1]
            for key, value in data.iteritems()
        }

    def from_multipart_form_data(self, request, **opts):
        from django.utils.datastructures import MultiValueDict

        # XXX not thrilled with this, but django itself uses an 'expect:' clause
        # to handle errors in this... so this is only slightly better...
        try:
            if hasattr(request, '_body'):
                data = StringIO(request._body)
            else:
                data = request
            post, files = request.parse_file_upload(request.META, data)
            data = MultiValueDict()
            data.update(post)
            data.update(files)
            return {
                key: value
                for key, value in data.iteritems()
            }
        except Exception, e:
            raise ValueError(str(e))

    def from_json(self, body, **opts):
        return json.loads(body)

    def get_content_type(self, format):
        '''get_content_type(format) -> content_type

        Look up a content type from a format.
        '''
        return self.format_to_content_type[format]

    def get_format(self, accept):
        '''get_format(content_type) -> format

        Find a suitable format from a content type.
        '''
        # ensure default_content_type is last so that accept = '*/*' picks up
        # our default_content_type
        content_types = self.content_types.keys()
        content_types.sort(key=lambda x: x == self.default_content_type)

        content_type = mimeparse.best_match(content_types, accept)
        if not content_type:
            raise self.Unsupported(accept)
        return content_type, self.content_types[content_type]

    def register(self, serializable):
        '''register(MyObject, SerializableMyObject(MyObject))

        Register a Serializable to use with a custom object.
        '''
        if serializable.klass in self.registered:
            raise self.SerializableExists('%s = %s' % (
                repr(serializable.klass),
                repr(self.registered[serializable.klass])
            ))
        self.registered[serializable.klass] = serializable

    def serialize(self, obj, accept=None, **opts):
        '''serialize(obj) -> content, content_type

        Serialize an object to text.
        '''
        accept = accept or self.default_content_type
        content_type, format = self.get_format(accept)
        method = getattr(self, 'to_%s' % format)

        # ugly hack to get the params to the header part out
        try:
            accept = [
                part
                for part in accept.split(',')
                if part.startswith(content_type)
            ][0]
        except IndexError:
            # '*/*' case
            accept = content_type

        params = mimeparse.parse_mime_type(accept)[2]
        for key, value in params.items():
            opts.setdefault(key, value)

        return self.SerializedContainer(method(obj, **opts), content_type)

    def to_json(self, obj, **opts):
        convert = self.convert
        dopts = {}

        if 'indent' in opts:
            try:
                dopts['indent'] = int(opts.pop('indent'))
            except ValueError:
                pass

        if opts:
            convert = partial(convert, **opts)

        return json.dumps(obj, default=convert, **dopts)



from django.db import models

class APITokenBase(models.Model):
    '''
    Use a token in place of user password to allow easy expiry.

    Example:

        class APIToken(APITokenBase):
            # required attributes
            #   - id (uniquely identifying the credential used)
            #   - user (either a user field or property returning the user)

            user = models.ForeignKey('auth.User')
            token = models.TextField()

            @classmethod
            def get(cls, owner, token):
                return cls.objects \
                    .select_related('user') \
                    .get(
                        user__username=owner,
                        token=token
                    )

            def is_valid(self):
                return self.user.is_active

            def save(self, *args, **kwargs):
                if not self.token:
                    self.token = ...


        from krankshaft import API as APIBase
        from krankshaft.auth import Auth as AuthBase
        from krankshaft.authn import AuthnDjangoAPIToken

        class Auth(AuthBase):
            authn = AuthnDjangoAPIToken(APIToken)

        class API(APIBase):
            Auth = Auth
    '''
    class Meta:
        abstract = True

    @classmethod
    def get(cls, owner, token):
        raise NotImplementedError

    def is_valid(self):
        '''is_valid() -> bool

        Test if token is still valid.
        '''
        return True



import time

class Throttle(object):
    '''
    Throttle requests based on authenticated client and a specified rate.

    The rate consists of the number of requests for a specific time period.

        rate = (number requests, time window/timedelta/seconds)

        ie: rate = (1000, timedelta(minutes=60))
            or rate = (1000, 300)

    The bucket determines the size of each "bucket" in which a request gets
    counted.  Instead of counting each request specifically, requests are
    grouped into bucket sized intervals (this is the minimum precision of when
    a request happened).  Bucket can be given as a timedelta() or seconds.
    Otherwise the bucket will default to the bucket_ratio of the time period
    of the rate.

    A suffix can be passed into various methods as a way to implement multiple
    throttles for a single authenticated user or for a single throttle per
    endpoint.

    There are also anon_bucket and anon_rate which are used for clients that
    are not authenticated.  By default, if only a rate is specified, the
    anonymous rate and bucket sizes are considerably smaller.  The reason being
    that by default, anonymous clients shouldn't have the same rate limits of
    an authenticated client or what would be the point of authenticating?

    The main goal of this implementation is to throttle requests in a constant
    time lookup so that the maximum number of requests doesnt define how long
    it takes to calculate if a request is allowed (which would be O(n)).  In
    this model, the lookup is constant time (which would be O(1)) based on the
    number of buckets (not the input number of requests).  This is required for
    a high preformance throttle.  The larger the bucket in comparison to the
    window, the fewer cache requests (the faster the lookup), but the longer it
    takes for a client over its quota to be able to make another request.
    '''
    anon_requests_ratio = 0.1
    anon_bucket_ratio = 0.1
    anon_rate = None
    bucket_ratio = 0.1
    format = 'throttle_%(id)s%(suffix)s'
    rate = None
    timer = time.time

    def __init__(self,
        request,
        auth,
        anon_bucket=None,
        anon_rate=None,
        bucket=None,
        cache=None,
        rate=None,
    ):
        self.auth = auth
        self.cache = cache or self.default_cache
        self.rate = rate or self.rate
        self.request = request

        self.bucket, self.rate = \
            self.make_bucket_rate(bucket, self.bucket_ratio, self.rate)

        anon_requests_ratio = self.anon_requests_ratio
        if anon_rate or self.anon_rate:
            anon_requests_ratio = 1
        self.anon_rate = anon_rate or self.anon_rate or self.rate
        self.anon_bucket, self.anon_rate = self.make_bucket_rate(
            anon_bucket,
            self.anon_bucket_ratio,
            self.anon_rate,
            anon_requests_ratio
        )

    def allow(self, suffix=None):
        '''allow() -> (bool, headers)

        Test if a request from a authenticated client is throttled.

        The headers returned should be added to the response when the request
        is not allowed.
        '''
        if self.auth.authned:
            bsize = self.bucket
            rate = self.rate
        else:
            bsize = self.anon_bucket
            rate = self.anon_rate

        if not rate:
            return (True, {})

        now = int(self.timer())
        key = self.key(suffix)
        nreq, nsec = rate

        current = now - (now % bsize)
        buckets = [current]
        first = now - nsec
        first = first - (first % bsize) # round down to previous bucket
        while current > first:
            current -= bsize
            buckets.append(current)

        buckets = [
            (bucket, key + '_b' + str(bucket))
            for bucket in buckets
        ]

        cached = self.cache.get_many([bkey for bucket, bkey in buckets])
        buckets = [
            (bucket, bkey, cached.get(bkey))
            for bucket, bkey in buckets
        ]

        requests_made = sum([
            bval
            for bucket, bkey, bval in buckets
            if bval
        ])
        bucket_current = buckets[0]

        if requests_made >= nreq:
            wait = None
            for bucket, bkey, bval in reversed(buckets):
                if not bval:
                    continue
                # ie: nsec = 10, bsize = 2, nreq = 1, now = 51, first = 42
                # buckets = (
                #   (40, None), # edge of window based on our bucket precision
                #   (42, None), # edge real window: 1 sec (42 - (now - nsec))
                #   (44, 1),    # edge real window: 3 sec (44 - (now - nsec))
                #   (46, None),
                #   (48, None),
                #   (50, None)
                # )
                # needs to wait 6 seconds to fall off end
                # wait(6) = bucket(44) - (now(51) - nsec(10)) + bsize(2) + 1
                #
                # need to wait the size of a single bucket on top of the
                # distance to the edge so that the bucket we're saying will
                # fall off actually falls off
                #
                # add one second to account for sub-second truncation,
                # otherwise, it's possible that a sub-second truncation error
                # will result in a client issuing a request after waiting for
                # the given seconds and being throttled
                #
                # however, to handle bucket's not being evenly disivible into
                # the nsec (rate time period) or that now minus nsec lands in
                # the middle of a bucket, we need to round down to the previous
                # bucket like:
                #
                # first = now - nsec               # edge of the real window
                # first = first - (first % bsize)  # round down to bucket
                #
                # so the formula becomes:
                # wait(7) = bucket(44) - first(40) + bsize(2) + 1
                # TODO this can be more accurate
                wait = bucket - first + bsize + 1
                break

            headers = {}
            if wait:
                headers['Retry-After'] = wait
            return (False, headers)

        else:
            try:
                self.cache.incr(bucket_current[1])
            except ValueError:
                # when incrementing a non-existant key on some cache backends
                self.cache.set(bucket_current[1], 1, self.timeout)
            return (True, {})

    @property
    def default_cache(self):
        from django.core.cache import cache
        return cache

    def make_bucket_rate(self, bucket, bucket_ratio, rate, requests_ratio=1):
        if bucket:
            if hasattr(bucket, 'total_seconds'):
                bucket = bucket.total_seconds()
            bucket = int(bucket)

        if rate:
            window = rate[1]
            if hasattr(window, 'total_seconds'):
                window = window.total_seconds()
            rate = (int(rate[0] * requests_ratio), int(window))

            if not bucket:
                bucket = int(rate[1] * bucket_ratio)

        return bucket, rate

    def key(self, suffix=None):
        '''key() -> key

        Construct a key for the authenticated client.
        '''
        return self.format % self.key_values(suffix)

    def key_values(self, suffix):
        return {
            'id': self.auth.id,
            'suffix': (suffix and ('_' + suffix) or ''),
        }

    @property
    def timeout(self):
        # its possible that if the current moment is exactly the same second
        # a bucket would be created, then the amount of buckets needed to
        # calculate is: (rate[1] / bucket) + 1
        #
        # extra bucket of time accounts for lag/latency/skew in time
        #
        # 3, 2 second buckets for a total 6 second window
        #   | | | | bucket markers
        #   ^     ^ first caret is beginning of window, second caret is now
        return self.rate[1] + (self.bucket * 2)



""" This code provides a decorator / context manager for transaction management in
    Django on PostgreSQL.  It is intended as a replacement for the existing Django
    commit_on_success() function, and provides some nice features:
    
    * Nested transactions: The top-level transaction will be a BEGIN/COMMIT/ROLLBACK
      block; inner "transactions" are implemented as savepoints.
    * Commits even if is_dirty is False, eliminating the mistake of forgetting to set
      the dirty flag when doing database-modifying raw SQL.
    * Better interaction with pgPool II, if you're using it.
    * A workaround for a subtle but nasty bug in Django's transaction management.
    
    For full details, check the README.md file.
"""

from functools import wraps

import psycopg2.extensions

from django.db import transaction, DEFAULT_DB_ALIAS, connections


class Rollback(Exception):
    """ This class provides a standard exception that can be thrown to handle a
        rollback condition.  If Xact receives an exception of this class (or
        a subclass), it swallows it and continues exception, rather than
        re-raising the exception.
    """
    pass

class _Transaction(object):

    """ This class manages a particular transaction or savepoint block, using context
        manager-style __enter__ and __exit__ statements.  We don't use it directly
        (for reasons noted below), but as a delegate for the _TransactionWrapper
        class.
    """
    
    def __init__(self, using):
        self.using = using
        self.sid = None
    
    def __enter__(self):
        if transaction.is_managed(self.using):
            # We're already in a transaction; create a savepoint.
            self.sid = transaction.savepoint(self.using)
        else:
            transaction.enter_transaction_management(using=self.using)
            transaction.managed(True, using=self.using)
   
    def __exit__(self, exc_type, exc_value, traceback):
        if exc_type is None:
            # commit operation
            if self.sid is None:
                # Outer transaction
                try:
                    transaction.commit(self.using)
                except:
                    transaction.rollback(self.using)
                    raise
                finally:
                    self._leave_transaction_management()
            else:
                # Inner savepoint
                try:
                    transaction.savepoint_commit(self.sid, self.using)
                except:
                    transaction.savepoint_rollback(self.sid, self.using)
                    raise
        else:
            # rollback operation
            if self.sid is None:
                # Outer transaction
                transaction.rollback(self.using)
                self._leave_transaction_management()
            else:
                # Inner savepoint
                transaction.savepoint_rollback(self.sid, self.using)
        
        if exc_type is None:
            return False
        else:
            return issubclass(exc_type, Rollback)
            # Returning False here means we did not gobble up the exception, so the
            # exception process should continue.
    
    def _leave_transaction_management(self):
        transaction.leave_transaction_management(using=self.using)
        if not connections[self.using].is_managed() and connections[self.using].features.uses_autocommit:
            connections[self.using]._set_isolation_level(psycopg2.extensions.ISOLATION_LEVEL_AUTOCOMMIT)
            # Patch for bug in Django's psycopg2 backend; see:
            # https://code.djangoproject.com/ticket/16047
    

class _TransactionWrapper:

    """ This class wraps the above _Transaction class.  We do this to allow reentrancy
        and thread-safety.  When being used as a decorator, only one _TransactionWrapper
        object is created per function being wrapped, and thus we can't store the state
        of the tranasction here (because multiple concurrent calls in the same address
        space to the same function would cause the state to be crunched), so delegate
        that to a _Transaction object that is created at the appropriate time.
        
        The "appropriate time" is two places: If the _TransactionWrapper is being used
        as a context manager, it's when the __enter__ function is called; if it is being
        used as a decorator, it's when the decorated function is about to be called
        (see the `inner` function below in __call__).
        
        The __enter__ and __exit__ functions on _TransactionWrapper are only called
        if we're using xact() as a context manager; if we're using it as a decorator,
        they're skipped and self.transaction is always None. Similarly, __call__ is
        not used if this is a context manager usage. This is not super-elegant, but
        it's the only way I've found to allow xact() to be used as both a context
        manager and a decorator using the same syntax.
    """
    
    def __init__(self, using):
        self.using = using
        self.transaction = None
    
    def __enter__(self):
        if self.transaction is None:
            self.transaction = _Transaction(self.using)
        return self.transaction.__enter__()
        
    def __exit__(self, exc_type, exc_value, traceback):
        return self.transaction.__exit__(exc_type, exc_value, traceback)

    def __call__(self, func):
        @wraps(func)
        def inner(*args, **kwargs):
            with _Transaction(self.using):
                return func(*args, **kwargs)
        return inner


def xact(using=None):
    if using is None:
        using = DEFAULT_DB_ALIAS
    if callable(using):
        # We end up here if xact is being used as a completely bare decorator:
        #   @xact
        # (not even an empty parameter list)
        return _TransactionWrapper(DEFAULT_DB_ALIAS)(using)
            # Note that `using` here is *not* the database alias; it's the actual function
            # being decorated.
    else:
        # We end up here if xact is being used as a parameterized decorator (including
        # default parameter):
        #    @xact(db)
        # or @xact()
        # ... or as a context manager:
        #    with xact():
        #       ...
        return _TransactionWrapper(using)
        


# -----------------------------------------------------------------------------
# This software is licensed under the PostgreSQL License:
#
#   http://www.postgresql.org/about/licence/
# 
# Copyright (c) 2012 Christophe Pettus
# 
# Permission to use, copy, modify, and distribute this software and its
# documentation for any purpose, without fee, and without a written agreement is
# hereby granted, provided that the above copyright notice and this paragraph
# and the following two paragraphs appear in all copies.
# 
# IN NO EVENT SHALL CHRISTOPHE PETTUS BE LIABLE TO ANY PARTY FOR DIRECT,
# INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES, INCLUDING LOST
# PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN
# IF CHRISTOPHE PETTUS HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH
# DAMAGE.
# 
# CHRISTOPHE PETTUS SPECIFICALLY DISCLAIMS ANY WARRANTIES, INCLUDING, BUT
# NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
# PARTICULAR PURPOSE. THE SOFTWARE PROVIDED HEREUNDER IS ON AN "AS IS" BASIS,
# AND CHRISTOPHE PETTUS HAS NO OBLIGATIONS TO PROVIDE MAINTENANCE,
# SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.



from dj_database_url import config as config_url
from django.core.urlresolvers import reverse_lazy
from os import path

PROJECT_ROOT = path.dirname(path.abspath(__file__))

CACHES = {
    'default': {
        'BACKEND': 'django.core.cache.backends.locmem.LocMemCache',
    }
}

DATABASES = {
    'default': config_url(default='postgres://localhost/krankshaft'),
}
DATABASES['default'].update({
    'OPTIONS': {
        'autocommit': True,
    }
})

TEMPLATE_DEBUG = DEBUG = True

ADMINS = MANAGERS = ()
APPEND_SLASH = False
LANGUAGE_CODE = 'en-us'
MEDIA_ROOT = path.join(PROJECT_ROOT, 'media')
MEDIA_URL = '/media/'
ROOT_URLCONF = 'tests.urls'
SECRET_KEY = 'a'
SITE_ID = 1
STATIC_ROOT = path.join(PROJECT_ROOT, '_static')
STATIC_URL = '/static/'
TIME_ZONE = 'America/Chicago'
USE_I18N = False
USE_L10N = False
USE_TZ = True
WSGI_APPLICATION = 'tests.wsgi.application'

STATICFILES_DIRS = (
    path.join(PROJECT_ROOT, 'static'),
)
TEMPLATE_DIRS = (
    path.join(PROJECT_ROOT, 'templates'),
)

STATICFILES_FINDERS = (
    'django.contrib.staticfiles.finders.FileSystemFinder',
    'django.contrib.staticfiles.finders.AppDirectoriesFinder',
#    'django.contrib.staticfiles.finders.DefaultStorageFinder',
)
TEMPLATE_LOADERS = (
    'django.template.loaders.filesystem.Loader',
    'django.template.loaders.app_directories.Loader',
)

INSTALLED_APPS = (
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.sites',
    'django.contrib.messages',
    'django.contrib.staticfiles',
    'django.contrib.admin',

    'krankshaft',
)

MIDDLEWARE_CLASSES = (
    'django.middleware.common.CommonMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    # 'django.middleware.clickjacking.XFrameOptionsMiddleware',
)

TEMPLATE_CONTEXT_PROCESSORS = (
    'django.contrib.auth.context_processors.auth',
    'django.core.context_processors.debug',
    #'django.core.context_processors.i18n',
    'django.core.context_processors.media',
    'django.core.context_processors.static',
    'django.core.context_processors.tz',
    'django.core.context_processors.request',
    'django.contrib.messages.context_processors.messages',
)

LOGGING = {
    'version': 1,
    'disable_existing_loggers': False,
    'filters': {
        'require_debug_false': {
            '()': 'django.utils.log.RequireDebugFalse'
        }
    },
    'handlers': {
        'mail_admins': {
            'level': 'ERROR',
            'filters': ['require_debug_false'],
            'class': 'django.utils.log.AdminEmailHandler'
        }
    },
    'loggers': {
        'django.request': {
            'handlers': ['mail_admins'],
            'level': 'ERROR',
            'propagate': True,
        },
    }
}



from __future__ import absolute_import

from django.core.handlers.wsgi import WSGIRequest
from django.test import TestCase
from django.test.client import MULTIPART_CONTENT, FakePayload
from django.utils.http import urlencode
from urlparse import urlparse

class TestCaseBase(TestCase):
    def make_request(self,
        method='GET',
        path='/',
        data=None,
        content_type=None,
        **extra
    ):
        if data is None:
            data = {}

        payload = None
        content_length = None
        if method not in ('DELETE', 'GET', 'HEAD', 'OPTIONS'):
            if content_type is None:
                content_type = MULTIPART_CONTENT
            encoded = self.client._encode_data(data, content_type)
            content_length = len(encoded)
            payload = FakePayload(encoded)
            data = {}

        if content_type is None:
            content_type = 'text/html; charset=utf-8'

        parsed = urlparse(path)
        environ = self.client._base_environ(**{
            'CONTENT_TYPE': content_type,
            'PATH_INFO': self.client._get_path(parsed),
            'QUERY_STRING': urlencode(data, doseq=True) or parsed[4],
            'REQUEST_METHOD': method,
        })

        if payload:
            environ['CONTENT_LENGTH'] = content_length
            environ['wsgi.input'] = payload

        environ.update(extra)
        return WSGIRequest(environ)

    def make_urlconf(self, *args, **kwargs):
        from django.conf.urls import patterns

        class Helper(object):
            pass

        module = Helper()
        module.urlpatterns = patterns('', *args, **kwargs)
        return module

class TestCaseNoDB(TestCaseBase):
    def _pre_setup(self):
        self.client = self.client_class()
        self._urlconf_setup()

    def _post_teardown(self):
        self._urlconf_teardown()



from .exceptions import InvalidOptions

class Annotate(object):
    '''
    with Annotate(request, {
        'auth': Auth(request),
        ...
    }, delete=['user']):
        ...

    Helper object to help with creating and cleaning up annotations on other
    objects.
    '''
    def __init__(self, obj, annotations, delete=None):
        self.obj = obj
        self.annotations = annotations
        self.delete = delete or []
        self.original = {}

    def __enter__(self):
        for name, value in self.annotations.iteritems():
            try:
                self.original[name] = getattr(self.obj, name)
            except AttributeError:
                pass

            setattr(self.obj, name, value)

        for name in self.delete:
            try:
                self.original[name] = getattr(self.obj, name)
                delattr(self.obj, name)
            except AttributeError:
                pass

    def __exit__(self, exc_type, exc_value, traceback):
        for name, value in self.annotations.iteritems():
            delattr(self.obj, name)

        for name, value in self.original.iteritems():
            setattr(self.obj, name, value)

def defaults(opts, *defaults):
    '''defaults({'a': 'a'}, {'a': 'b', 'b': 'b'}) -> {'a': 'a', 'b': 'b'}

    Set defaults in a dictionary.
    '''
    for default in defaults:
        for key, value in default.iteritems():
            if key not in opts:
                opts[key] = value
    return opts

def kw_as_header(kw):
    '''kw_as_header('Content_Type') -> Content-Type

    Convert a Python Keyword argument name to a valid Header name.  Since some
    characters used in headers are not valid Python Keyword characters, this
    serves as a translation for convenience.
    '''
    return kw.replace('_', '-')

def valid(opts, allowed):
    '''valid({'key': 'value'}, 'opt') -> raise InvalidOptions

    Validate that a dictionary has only valid options/keys.
    '''
    allowed = set(allowed)
    used = set(opts.keys())
    invalid = used - allowed
    if invalid:
        raise InvalidOptions(', '.join(sorted(list(invalid))))
    return opts
